"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ValueRange = /** @class */ (function () {
    function ValueRange(range) {
        var _a = tuple(range), min = _a[0], max = _a[1];
        this.min = min;
        this.max = max;
    }
    ValueRange.isZero = function (value) {
        var _a = tuple(value), min = _a[0], max = _a[1];
        return min === 0 && max === 0;
    };
    /**
     * compares the max value of two value ranges
     * @param a
     * @param b
     */
    ValueRange.cmpMax = function (a, b) {
        var a_max = Math.max.apply(Math, tuple(a));
        var b_max = Math.max.apply(Math, tuple(b));
        return a_max - b_max;
    };
    ValueRange.prototype.add = function (other) {
        if (other instanceof ValueRange && other.isAddIdentity()) {
            return this;
        }
        else {
            var _a = tuple(other), min = _a[0], max = _a[1];
            return new ValueRange([this.min + min, this.max + max]);
        }
    };
    ValueRange.prototype.mult = function (other) {
        if (other instanceof ValueRange && other.isMultIdentity()) {
            return this;
        }
        else {
            var _a = tuple(other), min = _a[0], max = _a[1];
            return new ValueRange([this.min * min, this.max * max]);
        }
    };
    ValueRange.prototype.map = function (mapFn) {
        var _a = [mapFn(this.min), mapFn(this.max)], min = _a[0], max = _a[1];
        if (min !== this.min || max !== this.max) {
            return new ValueRange([min, max]);
        }
        else {
            return this;
        }
    };
    ValueRange.prototype.isAddIdentity = function () {
        return this.min === 0 && this.max === 0;
    };
    ValueRange.prototype.isMultIdentity = function () {
        return this.min === 1 && this.max === 1;
    };
    /**
     * +29% => 1.29
     */
    ValueRange.prototype.percentToFactor = function () {
        return this.mult(new ValueRange([0.01, 0.01])).add(new ValueRange([1, 1]));
    };
    ValueRange.prototype.asTuple = function () {
        return [this.min, this.max];
    };
    ValueRange.prototype.valueOf = function () {
        var range = this.asTuple();
        if (range[0] === range[1]) {
            return range[0];
        }
        else {
            return range;
        }
    };
    return ValueRange;
}());
exports.default = ValueRange;
function tuple(value) {
    if (value instanceof ValueRange) {
        return value.asTuple();
    }
    else if (Array.isArray(value)) {
        return value;
    }
    else {
        return [value, value];
    }
}
// since ValueRange is immutable we can have this singletons
// can still be reassigned though :(
ValueRange.zero = new ValueRange([0, 0]);
ValueRange.one = new ValueRange([1, 1]);
