"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Mod_1 = require("../mods/Mod");
var ImmutableContainer_1 = require("./ImmutableContainer");
exports.SEXTANT_RANGE = 55; // http://poecraft.com/atlas has 55
var AtlasNode = /** @class */ (function (_super) {
    __extends(AtlasNode, _super);
    function AtlasNode(mods, props) {
        var _this = _super.call(this, mods) || this;
        _this.props = props;
        return _this;
    }
    AtlasNode.build = function (props) {
        return new AtlasNode([], props);
    };
    AtlasNode.withBuilder = function (builder) {
        return new AtlasNode(builder.mods, builder.props);
    };
    AtlasNode.prototype.builder = function () {
        return {
            mods: this.mods,
            props: this.props,
        };
    };
    /**
     * returns a list of nodes with sextant distance <= depth
     * sextant distance:
     * (a, b) => 1, if a.isInSextantRange(b)
     *           +inf, otherwise
     */
    AtlasNode.prototype.inSextantRange = function (atlas, max_depth) {
        if (max_depth === void 0) { max_depth = 1; }
        var expand = [this];
        var in_range = new Set();
        var depth = max_depth;
        // with every step mark every node in range as to be expanded and
        // decrement depth
        /* eslint-disable no-loop-func */
        while (expand.length && depth >= 0) {
            expand = expand.reduce(function (nodes, node) {
                in_range.add(node);
                var new_nodes = atlas.filter(function (new_node) {
                    return node.isInSextantRange(new_node) &&
                        node !== new_node &&
                        !in_range.has(new_node);
                });
                return nodes.concat(new_nodes);
            }, []);
            depth -= 1;
        }
        /* eslint-enable no-loop-func */
        return Array.from(in_range.values());
    };
    AtlasNode.prototype.isInSextantRange = function (other) {
        return this.distance(other) <= exports.SEXTANT_RANGE;
    };
    AtlasNode.prototype.distance = function (other) {
        return Math.sqrt(Math.pow((this.props.x - other.props.x), 2) + Math.pow((this.props.y - other.props.y), 2));
    };
    AtlasNode.prototype.pos = function () {
        return "X: " + this.props.x + " Y: " + this.props.y;
    };
    AtlasNode.prototype.getAllMods = function (atlas) {
        return this.inSextantRange(atlas).reduce(function (mods, other) { return mods.concat(other.mods); }, this.mods);
    };
    AtlasNode.prototype.getTags = function () {
        return _super.prototype.getTags.call(this)
            .concat(this.props.world_area.tags)
            .filter(
        // unique by id
        function (tag, i, tags) { return tags.findIndex(function (other) { return other === tag; }) === i; });
    };
    AtlasNode.prototype.maxModsOfType = function () {
        return Number.POSITIVE_INFINITY;
    };
    AtlasNode.prototype.inDomainOf = function (mod_domain) {
        return mod_domain === Mod_1.default.DOMAIN.ATLAS;
    };
    AtlasNode.prototype.level = function () {
        return this.props.world_area.area_level;
    };
    AtlasNode.prototype.affectingMods = function (atlas) {
        var _this = this;
        return atlas.reduce(function (mods, other) {
            if (_this.isInSextantRange(other)) {
                return mods.concat(other.mods);
            }
            else {
                return mods;
            }
        }, []); // this.mods will be passed on atlas.reduce
    };
    AtlasNode.prototype.activeMods = function (atlas) {
        var _this = this;
        return this.affectingMods(atlas).filter(function (mod) { return mod.spawnweightFor(_this) > 0; });
    };
    AtlasNode.prototype.inactiveMods = function (atlas) {
        var _this = this;
        return this.affectingMods(atlas).filter(function (mod) { return mod.spawnweightFor(_this) <= 0; });
    };
    return AtlasNode;
}(ImmutableContainer_1.default));
exports.default = AtlasNode;
