import { Tag } from '../schema';
import Mod from '../mods/Mod';
import Stat from '../calculator/Stat';
import Container from './Container';
/**
 * represents all the stats provided from the mods within the container
 * grouped by the stat id and their values added
 */
export interface StatWithModIndices {
    stat: Stat;
    mod_indices: number[];
}
export interface Builder<T extends Mod> {
    mods: T[];
}
/**
 * immutable implementation of Container
 */
export default abstract class ImmutableContainer<T extends Mod, B extends Builder<T>> implements Container<T> {
    mods: T[];
    constructor(mods: T[]);
    builder(): B;
    withMutations(mutate: (builder: B) => B): this;
    /**
     *  adds a new non-existing mod
     */
    addMod(mod: T): this;
    /**
     * truncates mods
     */
    removeAllMods(): this;
    /**
     * removes an existing mod
     */
    removeMod(other: T): this;
    indexOfModWithId(id: string): number;
    indexOfMod(mod: T): number;
    hasMod(mod: T): boolean;
    hasModGroup(other: T): boolean;
    /**
     * tags of the mods in the container
     */
    getTags(): Tag[];
    asArray(): T[];
    /**
     * @param {number} mod_type generation type
     */
    numberOfModsOfType(mod_type: number): number;
    /**
     * checks if theres more place for a mod with their generationtype
     */
    hasRoomFor(mod: T): boolean;
    /**
     * checks if this container has any mods
     */
    any(): boolean;
    /**
     * lists all the stats that are offered by its mods
     *
     * mods can have multiple stats so we sum their values grouped by stat id
     */
    stats(): {
        [key: string]: Stat;
    };
    statsExtended(): StatWithModIndices[];
    abstract maxModsOfType(mod: T): number;
    abstract inDomainOf(mod_domain: number): boolean;
    abstract level(): number;
    private statsExtendeMap;
}
