"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var Stat_1 = require("../calculator/Stat");
/**
 * immutable implementation of Container
 */
var ImmutableContainer = /** @class */ (function () {
    function ImmutableContainer(mods) {
        this.mods = mods;
    }
    ImmutableContainer.prototype.builder = function () {
        return { mods: this.mods };
    };
    // batch mutations
    ImmutableContainer.prototype.withMutations = function (mutate) {
        var builder = mutate(this.builder());
        // @ts-ignore
        return this.constructor.withBuilder(builder);
    };
    /**
     *  adds a new non-existing mod
     */
    ImmutableContainer.prototype.addMod = function (mod) {
        if (!this.hasMod(mod)) {
            return this.withMutations(function (builder) {
                return Object.assign({}, builder, { mods: builder.mods.concat(mod) });
            });
        }
        else {
            return this;
        }
    };
    /**
     * truncates mods
     */
    ImmutableContainer.prototype.removeAllMods = function () {
        if (this.mods.length > 0) {
            return this.withMutations(function (builder) {
                return Object.assign({}, builder, { mods: [] });
            });
        }
        else {
            return this;
        }
    };
    /**
     * removes an existing mod
     */
    ImmutableContainer.prototype.removeMod = function (other) {
        var _this = this;
        if (this.hasMod(other)) {
            return this.withMutations(function (builder) {
                return Object.assign({}, builder, {
                    mods: _this.mods.filter(function (mod) { return mod.props.id !== other.props.id; }),
                });
            });
        }
        else {
            return this;
        }
    };
    ImmutableContainer.prototype.indexOfModWithId = function (id) {
        return this.mods.findIndex(function (mod) { return mod.props.id === id; });
    };
    ImmutableContainer.prototype.indexOfMod = function (mod) {
        return this.indexOfModWithId(mod.props.id);
    };
    ImmutableContainer.prototype.hasMod = function (mod) {
        return this.indexOfMod(mod) !== -1;
    };
    ImmutableContainer.prototype.hasModGroup = function (other) {
        return (this.mods.find(function (mod) { return mod.props.correct_group === other.props.correct_group; }) !== undefined);
    };
    /**
     * tags of the mods in the container
     */
    ImmutableContainer.prototype.getTags = function () {
        return this.mods
            .reduce(function (tags, mod) {
            return tags.concat(mod.props.tags);
        }, [])
            .filter(
        // unique by id
        function (tag, i, tags) { return tags.findIndex(function (other) { return other === tag; }) === i; });
    };
    ImmutableContainer.prototype.asArray = function () {
        return this.mods;
    };
    /**
     * @param {number} mod_type generation type
     */
    ImmutableContainer.prototype.numberOfModsOfType = function (mod_type) {
        return this.mods.filter(function (mod) { return mod.props.generation_type === mod_type; })
            .length;
    };
    /**
     * checks if theres more place for a mod with their generationtype
     */
    ImmutableContainer.prototype.hasRoomFor = function (mod) {
        return (this.numberOfModsOfType(mod.props.generation_type) <
            this.maxModsOfType(mod));
    };
    /**
     * checks if this container has any mods
     */
    ImmutableContainer.prototype.any = function () {
        return this.mods.length > 0;
    };
    /**
     * lists all the stats that are offered by its mods
     *
     * mods can have multiple stats so we sum their values grouped by stat id
     */
    ImmutableContainer.prototype.stats = function () {
        return lodash_1.mapValues(this.statsExtendeMap(), function (_a) {
            var stat = _a.stat;
            return stat;
        });
    };
    ImmutableContainer.prototype.statsExtended = function () {
        return Object.values(this.statsExtendeMap());
    };
    ImmutableContainer.prototype.statsExtendeMap = function () {
        return this.mods.reduce(function (stats, mod, index) {
            // flattened
            return mod.statsJoined().reduce(function (joined, stat) {
                var _a, _b;
                var id = stat.props.id;
                var existing = joined[id];
                // group by stat.Id
                if (existing && existing.stat instanceof Stat_1.default) {
                    return __assign({}, joined, (_a = {}, _a[id] = {
                        stat: existing.stat.add(stat.values),
                        mod_indices: existing.mod_indices.concat(index),
                    }, _a));
                }
                else {
                    return __assign({}, joined, (_b = {}, _b[id] = {
                        stat: stat,
                        mod_indices: [index],
                    }, _b));
                }
            }, stats);
        }, {});
    };
    return ImmutableContainer;
}());
exports.default = ImmutableContainer;
