"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var RarityKind;
(function (RarityKind) {
    RarityKind[RarityKind["normal"] = 1] = "normal";
    RarityKind[RarityKind["magic"] = 2] = "magic";
    RarityKind[RarityKind["rare"] = 3] = "rare";
    RarityKind[RarityKind["unique"] = 4] = "unique";
    RarityKind[RarityKind["showcase"] = 5] = "showcase";
})(RarityKind = exports.RarityKind || (exports.RarityKind = {}));
/**
 * the rarity of an item
 */
var ItemRarity = /** @class */ (function () {
    function ItemRarity(item, builder) {
        this.parent = item;
        this.kind = builder;
    }
    ItemRarity.prototype.builder = function () {
        return this.kind;
    };
    ItemRarity.prototype.isNormal = function () {
        return this.kind === RarityKind.normal;
    };
    ItemRarity.prototype.isMagic = function () {
        return this.kind === RarityKind.magic;
    };
    ItemRarity.prototype.isRare = function () {
        return this.kind === RarityKind.rare || this.kind === RarityKind.showcase;
    };
    ItemRarity.prototype.isUnique = function () {
        return this.kind === RarityKind.unique;
    };
    /**
     * upgrade rarirty by one tier
     *
     * normal > magic > rare
     */
    ItemRarity.prototype.upgrade = function () {
        var new_rarity = this.kind;
        if (this.isNormal()) {
            new_rarity = RarityKind.magic;
        }
        else if (this.isMagic()) {
            new_rarity = RarityKind.rare;
        }
        return this.parent.withMutations(function (builder) {
            return __assign({}, builder, { rarity: new_rarity });
        });
    };
    ItemRarity.prototype.set = function (rarity) {
        return this.parent.withMutations(function (builder) {
            return __assign({}, builder, { rarity: RarityKind[rarity] });
        });
    };
    ItemRarity.prototype.toString = function () {
        return RarityKind[this.kind].toString();
    };
    ItemRarity.prototype.any = function () {
        // ItemRarity always has a rarity
        return true;
    };
    return ItemRarity;
}());
exports.default = ItemRarity;
