"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * the requirements to use this item
 *
 * contains attributes strength, intelligence, evasion
 * and the itemlevel
 */
var ItemName = /** @class */ (function () {
    function ItemName(item, builder) {
        this.dex = 0;
        this.int = 0;
        this.str = 0;
        this.parent = item;
        if (builder != null) {
            this.dex = builder.req_dex;
            this.int = builder.req_int;
            this.str = builder.req_str;
        }
    }
    ItemName.prototype.builder = function () {
        return {
            req_dex: this.dex,
            req_int: this.int,
            req_str: this.str,
        };
    };
    /**
     * Computes the required level including stat applications
     */
    ItemName.prototype.level = function () {
        return this.parent.computeValue(this.baseLevel(), [
            'local',
            'requirements',
            'level',
        ]);
    };
    ItemName.prototype.list = function () {
        var attribute_requirements_classification = ['local', 'requirements'];
        return {
            level: this.level(),
            str: this.parent.computeValue(this.str, attribute_requirements_classification.concat([
                'strength',
            ])),
            dex: this.parent.computeValue(this.dex, attribute_requirements_classification.concat([
                'dexterity',
            ])),
            int: this.parent.computeValue(this.int, attribute_requirements_classification.concat([
                'intelligence',
            ])),
        };
    };
    ItemName.prototype.any = function () {
        return Object.values(this.list()).some(function (_a) {
            var value = _a.value;
            return value !== 0;
        });
    };
    /**
     * computes required level before stat applications
     *
     * @returns number
     */
    ItemName.prototype.baseLevel = function () {
        if (this.parent.meta_data.isA('AbstractMap')) {
            return 0;
        }
        else {
            return Math.max.apply(Math, [this.parent.baseitem.drop_level].concat(this.parent.mods.map(function (mod) { return mod.requiredLevel(); })));
        }
    };
    return ItemName;
}());
exports.default = ItemName;
