import Item from '../Item';
import { RarityIdent } from '../components/Rarity';
import { AugmentableValue } from '../util';
import { ModGroup } from './types';
import { BaseItemTypeProps, ModProps } from '../../../schema';
import { Socket, SocketGroup } from '../components/Sockets';
/**
 * a serialization of an {Item} that aims convey meaning of its properties
 * and not how they are supposed to be displayed.
 *
 * Its goal is it to be deserialized into an Item
 */
export interface SemanticItemJSON {
    base: BaseItemTypeProps;
    name: string;
    rarity: RarityIdent;
    requirements: {
        level: AugmentableValue;
        dex: AugmentableValue;
        int: AugmentableValue;
        str: AugmentableValue;
    };
    mods?: {
        [key in ModGroup]?: ModProps[];
    };
    elder?: boolean;
    shaper?: boolean;
    corrupted?: boolean;
    mirrored?: boolean;
    item_level?: number;
    quality?: number;
    sockets: Socket[];
    groups: SocketGroup[];
}
export declare function serialize(item: Item): SemanticItemJSON;
export declare function deserialize(json: SemanticItemJSON): Item;
