"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var meta_mods_1 = require("../mods/meta_mods");
var Flags_1 = require("../util/Flags");
/**
 * @abstract
 */
var Generator = /** @class */ (function () {
    function Generator(mods) {
        this.mods = mods;
    }
    /**
     * returns a copy of #mods
     *
     * we can stick with a shallow copy since Mod are supposed to be immutable
     */
    Generator.prototype.getAvailableMods = function () {
        return this.mods.slice();
    };
    Generator.prototype.isApplicableTo = function (container, whitelist) {
        if (whitelist === void 0) { whitelist = []; }
        return !Flags_1.anySet(this.applicableTo(container), whitelist);
    };
    Generator.prototype.isModApplicableTo = function (mod, container) {
        var applicable_flags = {
            domain_full: false,
            already_present: false,
            wrong_domain: false,
            lower_ilvl: false,
            above_lld_level: false,
        };
        if (!container.inDomainOf(mod.props.domain)) {
            applicable_flags.wrong_domain = true;
        }
        else if (!container.hasRoomFor(mod)) {
            applicable_flags.domain_full = true;
        }
        if (mod.props.level > container.level()) {
            applicable_flags.lower_ilvl = true;
        }
        if (container.hasModGroup(mod)) {
            applicable_flags.already_present = true;
        }
        var has_leo_meta_mod = container.indexOfModWithId(meta_mods_1.default.LLD_MOD) !== -1;
        if (mod.requiredLevel() > 28 && has_leo_meta_mod) {
            applicable_flags.above_lld_level = true;
        }
        return applicable_flags;
    };
    return Generator;
}());
exports.default = Generator;
