"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var mods_1 = require("../mods");
var Flags_1 = require("../util/Flags");
var rng_1 = require("../util/rng");
var Generator_1 = require("./Generator");
/**
 * @abstract
 * a Generator that randomly rolls one of its mods
 * ignores mods that have no spawnweight or 0 spawnweight for every tag
 */
var Orb = /** @class */ (function (_super) {
    __extends(Orb, _super);
    function Orb() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Orb.modFilter = function (mod) {
        return (mod.spawn_weights.length > 0 &&
            mod.spawn_weights.some(function (_a) {
                var value = _a.value;
                return value > 0;
            }));
    };
    Orb.buildMods = function (mods) {
        var _this = this;
        return mods
            .filter(function (props) { return _this.modFilter(props); })
            .map(function (props) { return new mods_1.Mod(props); });
    };
    Orb.prototype.chooseMod = function (container) {
        var details = this.modsFor(container);
        var detail = rng_1.choose(details, function (other) {
            if (other.spawnweight == null) {
                throw new Error('optional spawnweight not allowed when choosing');
            }
            return other.spawnweight;
        });
        if (detail != null) {
            return detail.mod;
        }
        else {
            return undefined;
        }
    };
    /**
     * adds a mod from chooseMod ignoring if it's applicable
     * @param {Item} item
     */
    Orb.prototype.rollMod = function (container) {
        var mod = this.chooseMod(container);
        if (mod != null) {
            return container.addMod(mod);
        }
        else {
            return container;
        }
    };
    Orb.prototype.isModSpawnableOn = function (mod, container) {
        var spawnable_flags = {
            no_matching_tags: false,
            spawnweight_zero: false,
        };
        var spawnweight = this.spawnweightFor(mod, container);
        if (spawnweight == null) {
            // at first glance this shouldn't be happening
            // since every mod seems to have a spawnweight for the default
            // tag which every equipment seems to have
            spawnable_flags.no_matching_tags = true;
        }
        else if (spawnweight <= 0) {
            spawnable_flags.spawnweight_zero = true;
        }
        return spawnable_flags;
    };
    Orb.prototype.spawnweightFor = function (mod, container) {
        var props = mod.spawnweightPropsFor(container);
        if (props == null) {
            return undefined;
        }
        else {
            return props.value;
        }
    };
    Orb.prototype.modsFor = function (container, whitelist) {
        var _this = this;
        if (whitelist === void 0) { whitelist = []; }
        var details = [];
        this.getAvailableMods().forEach(function (mod) {
            var applicable_flags = _this.isModApplicableTo(mod, container);
            var spawnable_flags = _this.isModSpawnableOn(mod, container);
            var spawnweight = _this.spawnweightFor(mod, container);
            var is_applicable = !Flags_1.anySet(applicable_flags, whitelist);
            var is_spawnable = !Flags_1.anySet(spawnable_flags, whitelist);
            var is_rollable = is_applicable && is_spawnable;
            if (is_rollable) {
                details.push({
                    mod: mod,
                    applicable: applicable_flags,
                    spawnable: spawnable_flags,
                    spawnweight: spawnweight,
                });
            }
        });
        return details;
    };
    return Orb;
}(Generator_1.default));
exports.default = Orb;
