import Item from '../../containers/item';
import { ModProps, EssenceProps } from '../../schema';
import ItemOrb, { ApplicableFlags as BaseApplicableFlags } from './ItemOrb';
import Mod from '../../mods/Mod';
import { GeneratorDetails } from '../Generator';
export interface ApplicableFlags extends BaseApplicableFlags {
    wrong_rarity: boolean;
    wrong_itemclass: boolean;
}
export declare type ApplicableFlag = keyof ApplicableFlags;
/**
 * Essences guarantee at least exactly one mod depending on itemclass
 * or are not applicable. They should provide a mod for every equipment type.
 */
export default class Essence extends ItemOrb {
    static build(props: EssenceProps, mods: ModProps[]): Essence;
    private static reforger;
    props: EssenceProps;
    private alchemy;
    /**
     *
     * @param props props of a specific essence (contains the guarenteed mod)
     * @param mods mods that are rolled onto the item after th guarenteed is applied
     */
    constructor(props: EssenceProps, mods: ModProps[]);
    /**
     *  add the guarenteed mod and fill up the rest like alchemy
     */
    applyTo(item: Item, options?: Partial<{
        force: boolean;
    }>): Item;
    /**
     * only one mod per itemclass
     */
    modsFor(item: Item, whitelist?: string[]): Array<GeneratorDetails<Mod>>;
    /**
     * @returns Mod if the itemclass of the Item is eligible
     */
    chooseMod(item: Item): Mod | undefined;
    /**
     *
     * @param itemclass
     * @returns the guaranteed mod for the itemclass
     */
    modForItemclass(itemclass: string): Mod | undefined;
    /**
     * applicable if the essence guarantees a mod for the itemclass
     * and the rarity is either white or rare (only if essence can reforge)
     * @param item
     */
    applicableTo(item: Item): ApplicableFlags;
    /**
     * @returns true if the essence can reforge (i.e. reroll) the item
     */
    reforges(): boolean;
    /**
     * @returns true if the essence is the best of its type
     */
    isTopTier(): boolean;
    /**
     * mapping from itemclass to mod prop in essence props
     * @param item_class
     */
    private modPropsFor;
}
