"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var ViewOnlyOrb_1 = require("../ViewOnlyOrb");
var mods_1 = require("../../mods");
/**
 * Generator for all mods that can be crafted at the Blood Altar with Spirit Beasts
 */
var IncursionTempleMods = /** @class */ (function (_super) {
    __extends(IncursionTempleMods, _super);
    function IncursionTempleMods() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    IncursionTempleMods.build = function (mods) {
        return new IncursionTempleMods(mods.filter(IncursionTempleMods.isTempleMod).map(function (props) { return new mods_1.Mod(props); }));
    };
    /**
     *
     * @param mod
     * @returns true if cab only be found in T3 temple chests
     */
    IncursionTempleMods.isTempleMod = function (mod) {
        return /Enhanced.*Mod/.test(mod.id);
    };
    /**
     * @override
     *
     * super call with the addition of item class check
     * mods are assigned to certain itemclasses which is determined
     * in {itemclassModIds()}
     *
     * @param mod
     * @param item
     * @returns {SpawnableFlags}
     */
    IncursionTempleMods.prototype.isModSpawnableOn = function (mod, item) {
        var mod_ids = this.itemclassModIds(item.baseitem
            .item_class);
        var not_matching_itemclass = !mod_ids.includes(mod.props.id);
        return __assign({}, _super.prototype.isModSpawnableOn.call(this, mod, item), { not_matching_itemclass: not_matching_itemclass });
    };
    /**
     * @override
     *
     * We assume that every mod has the same chance
     *
     * @param mod
     * @param item
     * @returns {number | undefined}
     */
    IncursionTempleMods.prototype.spawnweightFor = function (mod, item) {
        // assume they are evenly distributed
        return 1000;
    };
    IncursionTempleMods.prototype.itemclassModIds = function (item_class) {
        switch (item_class) {
            case 'Helmet':
                return [
                    // of Puhuarte
                    'FireResistEnhancedModPhys_',
                    'ColdResistEnhancedModPhys',
                    'LightningResistEnhancedModPhys',
                    // Xopec's
                    'IncreasedManaEnhancedModPercent',
                    'IncreasedManaEnhancedModReservation',
                ];
            case 'Boots':
                return [
                    // Matatl's
                    'MovementVelocityEnhancedModSpeed',
                    'MovementVelocityEnhancedModDodge_',
                    'MovementVelocityEnhancedModSpellDodge_',
                    // Xopec's
                    'IncreasedManaEnhancedModPercent',
                ];
            case 'Gloves':
                return [
                    // Xopec's
                    'IncreasedManaEnhancedModPercent',
                    // Topotante's
                    'AddedFireDamageEnhancedMod',
                    'AddedColdDamageEnhancedMod',
                    'AddedLightningDamageEnhancedMod',
                    // of Puhuarte
                    'FireResistEnhancedModAilments',
                    'ColdResistEnhancedModAilments__',
                    'LightningResistEnhancedModAilments',
                ];
            case 'Body Armour':
                return [
                    // of Tacati
                    'ChaosResistEnhancedMod_',
                    // Guatelitzi's
                    'IncreasedLifeEnhancedBodyMod___',
                ];
            case 'Shield':
                return [
                    // of Tacati
                    'ChaosResistEnhancedMod_',
                    // Topotante's
                    'FireDamagePrefixOnWeaponEnhancedMod',
                    'ColdDamagePrefixOnWeaponEnhancedMod',
                    'LightningDamagePrefixOnWeaponEnhancedMod_',
                ];
            case 'Dagger':
                return [
                    // Tacati's
                    'LocalIncreasedPhysicalDamageEnhancedMod',
                    // of Tacati
                    'LocalIncreasedAttackSpeedEnhancedMod',
                    'PoisonDurationEnhancedMod',
                    'ChanceToPoisonEnhancedMod',
                    'PoisonDamageEnhancedAttacksMod',
                    'PoisonDamageEnhancedSpellsMod',
                    // Topotante's
                    'LocalAddedFireDamageEnhancedMod_',
                    'LocalAddedColdDamageEnhancedMod',
                    'LocalAddedLightningDamageEnhancedMod',
                    // Matatl's
                    'TrapDamageOnWeaponEnhancedMod',
                    'LocalIncreaseSocketedTrapGemLevelEnhancedMod_',
                    // of Matatl
                    'TrapThrowSpeedEnhancedMod',
                    'TrapCooldownRecoveryAndDurationEnhancedMod',
                    'TrapAreaOfEffectEnhancedMod_',
                ];
            case 'Claw':
            case 'Thrusting One Hand Sword':
            case 'One Hand Sword':
                return [
                    // Topotante's
                    'LocalAddedFireDamageEnhancedMod_',
                    'LocalAddedColdDamageEnhancedMod',
                    'LocalAddedLightningDamageEnhancedMod',
                    // of Tacati
                    'LocalIncreasedAttackSpeedEnhancedMod',
                    'PoisonDurationEnhancedMod',
                    'ChanceToPoisonEnhancedMod',
                    'PoisonDamageEnhancedAttacksMod',
                    // Tacati's
                    'LocalIncreasedPhysicalDamageEnhancedMod',
                ];
            case 'One Hand Mace':
            case 'One Hand Axe':
                return [
                    // Topotante's
                    'LocalAddedFireDamageEnhancedMod_',
                    'LocalAddedColdDamageEnhancedMod',
                    'LocalAddedLightningDamageEnhancedMod',
                    // of Tacati
                    'LocalIncreasedAttackSpeedEnhancedMod',
                    // Tacati's
                    'LocalIncreasedPhysicalDamageEnhancedMod',
                ];
            case 'Sceptre':
            case 'Wand':
                return [
                    // Tacati's
                    'LocalIncreasedPhysicalDamageEnhancedMod',
                    'SpellDamageOnWeaponEnhancedMod',
                    // of Tacati
                    'IncreasedCastSpeedEnhancedMod_',
                    'LocalIncreasedAttackSpeedRangedEnhancedMod_',
                    // Topotante's
                    'LocalAddedFireDamageEnhancedMod_',
                    'LocalAddedColdDamageEnhancedMod',
                    'LocalAddedLightningDamageEnhancedMod',
                    'FireDamagePrefixOnWeaponEnhancedMod',
                    'ColdDamagePrefixOnWeaponEnhancedMod',
                    'LightningDamagePrefixOnWeaponEnhancedMod_',
                    // Matatl's
                    'TrapDamageOnWeaponEnhancedMod',
                    'LocalIncreaseSocketedTrapGemLevelEnhancedMod_',
                    // of Matatl
                    'TrapThrowSpeedEnhancedMod',
                    'TrapCooldownRecoveryAndDurationEnhancedMod',
                    'TrapAreaOfEffectEnhancedMod_',
                    // Citaqualotl's
                    'MinionDamageOnWeaponEnhancedMod__',
                    'LocalIncreaseSocketedMinionGemLevelEnhancedMod',
                    // of of Citaqualotl
                    'MinionAttackAndCastSpeedEnhancedMod',
                    'MinionDurationEnhancedMod_',
                ];
            case 'Two Hand Axe':
            case 'Two Hand Mace':
                return [
                    // Tacati's
                    'LocalIncreasedPhysicalDamageEnhancedMod',
                    // of Tacati
                    'LocalIncreasedAttackSpeedEnhancedMod',
                    // Topotante's
                    'LocalAddedFireDamageEnhancedTwoHandMod',
                    'LocalAddedColdDamageEnhancedTwoHandMod',
                    'LocalAddedLightningDamageEnhancedTwoHandMod',
                ];
            case 'Two Hand Sword':
                return [
                    // Tacati's
                    'LocalIncreasedPhysicalDamageEnhancedMod',
                    // of Tacati
                    'LocalIncreasedAttackSpeedEnhancedMod',
                    'PoisonDurationEnhancedMod',
                    'ChanceToPoisonEnhancedMod',
                    'PoisonDamageEnhancedAttacksMod',
                    // Topotante's
                    'LocalAddedFireDamageEnhancedTwoHandMod',
                    'LocalAddedColdDamageEnhancedTwoHandMod',
                    'LocalAddedLightningDamageEnhancedTwoHandMod',
                ];
            case 'Bow':
                return [
                    // Tacati's
                    'LocalIncreasedPhysicalDamageEnhancedMod',
                    // of Tacati
                    'LocalIncreasedAttackSpeedRangedEnhancedMod_',
                    'PoisonDurationEnhancedMod',
                    'ChanceToPoisonEnhancedMod',
                    'PoisonDamageEnhancedAttacksMod',
                    // Topotante's
                    'LocalAddedFireDamageEnhancedTwoHandMod',
                    'LocalAddedColdDamageEnhancedTwoHandMod',
                    'LocalAddedLightningDamageEnhancedTwoHandMod',
                ];
            case 'Staff':
                return [
                    // Tacati's
                    'LocalIncreasedPhysicalDamageEnhancedMod',
                    'SpellDamageOnTwoHandWeaponEnhancedMod',
                    // of Tacati
                    'IncreasedCastSpeedTwoHandEnhancedMod',
                    // Topotante's
                    'LocalAddedFireDamageEnhancedTwoHandMod',
                    'LocalAddedColdDamageEnhancedTwoHandMod',
                    'LocalAddedLightningDamageEnhancedTwoHandMod',
                    'FireDamagePrefixOnTwoHandWeaponEnhancedMod',
                    'ColdDamagePrefixOnTwoHandWeaponEnhancedMod',
                    'LightningDamagePrefixOnTwoHandWeaponEnhancedMod',
                    // Matatl's
                    'TrapDamageOnTwoHandWeaponEnhancedMod',
                    'LocalIncreaseSocketedTrapGemLevelEnhancedMod_',
                    // of Matatl
                    'TrapThrowSpeedEnhancedMod',
                    'TrapCooldownRecoveryAndDurationEnhancedMod',
                    'TrapAreaOfEffectEnhancedMod_',
                    // Citaqualotl's
                    'MinionDamageOnTwoHandWeaponEnhancedMod',
                    'LocalIncreaseSocketedMinionGemLevelEnhancedMod',
                    // of of Citaqualotl
                    'MinionAttackAndCastSpeedEnhancedMod',
                    'MinionDurationEnhancedMod_',
                ];
            case 'Amulet':
                return [
                    // Xopec's
                    'IncreasedManaEnhancedModOnHit_',
                    'IncreasedManaEnhancedModRegen',
                    // of Puhuarte
                    'FireResistEnhancedModLeech',
                    'ColdResistEnhancedModLeech',
                    'LightningResistEnhancedModLeech_',
                    // Guatelitzi's
                    'IncreasedLifeEnhancedMod',
                    'IncreasedEnergyShieldEnhancedModES',
                    'IncreasedEnergyShieldEnhancedModRegen_',
                    // of Guatelitzi
                    'LifeRegenerationEnhancedMod',
                ];
            case 'Ring':
                return [
                    // Xopec's
                    'IncreasedManaEnhancedModOnHit_',
                    'IncreasedManaEnhancedModRegen',
                    'IncreasedManaEnhancedModCost',
                    // Guatelitzi's
                    'IncreasedLifeEnhancedMod',
                    'IncreasedEnergyShieldEnhancedModES',
                    'IncreasedEnergyShieldEnhancedModRegen_',
                    // of Guatelitzi
                    'LifeRegenerationEnhancedMod',
                ];
            case 'Belt':
                return [
                    // Guatelitzi's
                    'IncreasedLifeEnhancedMod',
                    'IncreasedEnergyShieldEnhancedModES',
                    'IncreasedEnergyShieldEnhancedModRegen_',
                    // of Guatelitzi
                    'LifeRegenerationEnhancedMod',
                ];
            default:
                return [];
        }
    };
    return IncursionTempleMods;
}(ViewOnlyOrb_1.default));
exports.default = IncursionTempleMods;
