import { GeneratorDetails } from '../Generator';
import ViewOnlyOrb from '../ViewOnlyOrb';
import Item from '../../containers/item';
import { Mod } from '../../mods';
import { ModProps } from '../../schema';
/**
 * Generator for all mods that can appear on items sold by Master vendors
 *
 * Remember that items sold from the vendor have a fixed item level and might
 * not be able to hold mods for higher item levels
 */
export default class MasterSignatureMods extends ViewOnlyOrb {
    /**
     * mapping from mod id to master id
     *
     * we need to know what master vendor can spawn the mods because the vendor
     * tells us what itemclasses can appear
     */
    static readonly MOD_MASTER: {
        [key: string]: string;
    };
    /**
     * mapping from master id to a filter function that returns true if it could've
     * sold the given item
     *
     * TODO required level? Do master sell the best bases e.g. Hubris Circlets
     */
    static readonly MASTER_ITEMCLASSES: {
        [key: string]: (item: Item) => boolean;
    };
    static build(mods: ModProps[]): MasterSignatureMods;
    /**
     *
     * @param mod
     * @returns true if the mod can only be found on items bought from masters
     */
    static isMasterSignature(mod: ModProps): boolean;
    /**
     * @param item
     * @param whitelist
     */
    modsFor(item: Item, whitelist?: string[]): Array<GeneratorDetails<Mod>>;
}
