import { GeneratorDetails } from '../Generator';
import ViewOnlyOrb from '../ViewOnlyOrb';
import Item from '../../containers/item';
import { Mod } from '../../mods';
import { ModProps } from '../../schema';
export declare type Faction = 'brinerot' | 'mutewind' | 'redblade' | 'renegade';
/**
 * Generator for all mods that can appear on items dropped by Warbands
 *
 * Each Faction of Warbands can drop a magic item that has a single Warbands
 * mod. The itemclass of that item depends on the faction
 */
export default class WarbandsMods extends ViewOnlyOrb {
    /**
     * mapping from Warband factions to their mods
     */
    static readonly FACTION_MODS: {
        [faction in Faction]: Set<Mod['props']['id']>;
    };
    /**
     * mapping from Warband factions to the item classes they can drop
     */
    static readonly FACTION_ITEM_CLASSES: {
        [faction in Faction]: (item: Item) => boolean;
    };
    static build(mods: ModProps[]): WarbandsMods;
    /**
     *
     * @param mod
     * @returns true if the mod can spawn on Warbands drops
     */
    static isWarbands(mod: ModProps): boolean;
    modsFor(item: Item, whitelist?: string[]): Array<GeneratorDetails<Mod>>;
    private factionOf;
}
