"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var ViewOnlyOrb_1 = require("../ViewOnlyOrb");
var mods_1 = require("../../mods");
/**
 * Generator for all mods that can appear on items dropped by Warbands
 *
 * Each Faction of Warbands can drop a magic item that has a single Warbands
 * mod. The itemclass of that item depends on the faction
 */
var WarbandsMods = /** @class */ (function (_super) {
    __extends(WarbandsMods, _super);
    function WarbandsMods() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WarbandsMods.build = function (mods) {
        return new WarbandsMods(mods.filter(WarbandsMods.isWarbands).map(function (props) { return new mods_1.Mod(props); }));
    };
    /**
     *
     * @param mod
     * @returns true if the mod can spawn on Warbands drops
     */
    WarbandsMods.isWarbands = function (mod) {
        return (Object.values(WarbandsMods.FACTION_MODS).find(function (mods) {
            return mods.has(mod.id);
        }) !== undefined);
    };
    WarbandsMods.prototype.modsFor = function (item, whitelist) {
        if (whitelist === void 0) { whitelist = []; }
        var faction = this.factionOf(item);
        if (faction === undefined) {
            return [];
        }
        // mod ids that are spawnable by the faction responsible for this itemclass
        var mod_ids = WarbandsMods.FACTION_MODS[faction];
        // omit everything spawnable related. crafting is deterministic
        return _super.prototype.modsFor.call(this, item, whitelist.concat('no_matching_tags', 'spawnweight_zero'))
            .map(function (_a) {
            var spawnable = _a.spawnable, spawnweight = _a.spawnweight, details = __rest(_a, ["spawnable", "spawnweight"]);
            return details;
        })
            .filter(function (_a) {
            var mod = _a.mod;
            return mod_ids.has(mod.props.id);
        });
    };
    WarbandsMods.prototype.factionOf = function (item) {
        var faction = Object.entries(WarbandsMods.FACTION_ITEM_CLASSES).find(function (_a) {
            var isDroppable = _a[1];
            return isDroppable(item);
        });
        if (faction === undefined) {
            return undefined;
        }
        else {
            return faction[0];
        }
    };
    /**
     * mapping from Warband factions to their mods
     */
    WarbandsMods.FACTION_MODS = {
        brinerot: new Set(['DamageDuringFlaskEffectWarbands']),
        mutewind: new Set(['CannotBeFrozenWarbands']),
        redblade: new Set(['PhysicalDamageTakenAsFirePercentWarbands']),
        renegade: new Set([
            'FireResistancePenetrationWarbands',
            'LightningPenetrationWarbands',
            'ColdResistancePenetrationWarbands',
        ]),
    };
    /**
     * mapping from Warband factions to the item classes they can drop
     */
    WarbandsMods.FACTION_ITEM_CLASSES = {
        brinerot: function (item) { return item.baseitem.item_class === 'Gloves'; },
        mutewind: function (item) { return item.baseitem.item_class === 'Boots'; },
        redblade: function (item) { return item.baseitem.item_class === 'Helmet'; },
        renegade: function (item) { return item.meta_data.isA('AbstractWeapon'); },
    };
    return WarbandsMods;
}(ViewOnlyOrb_1.default));
exports.default = WarbandsMods;
