"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var AtlasNode_1 = require("../containers/AtlasNode");
var Sextant_1 = require("../generators/Sextant");
/**
 * immutable data structure for the atlas in Path of Exile
 *
 * main purpose is for reducer like usage in redux
 */
var Atlas = /** @class */ (function () {
    function Atlas(nodes) {
        this.nodes = nodes;
    }
    Atlas.buildLookupTable = function (atlas) {
        return atlas.reduce(function (nodes, props) {
            return nodes.set(props.world_area.id, new AtlasNode_1.default([], props));
        }, new Map());
    };
    Atlas.build = function (atlas) {
        var nodes = Atlas.buildLookupTable(atlas);
        return new Atlas(nodes);
    };
    Atlas.withBuilder = function (builder) {
        return new Atlas(builder.nodes);
    };
    // TODO in the future use [Symbol.iterator]()
    Atlas.prototype.asArray = function () {
        return Array.from(this.nodes.values());
    };
    /**
     * wrapper for map get that ensures a node or throws
     */
    Atlas.prototype.get = function (world_area_id) {
        var node = this.nodes.get(world_area_id);
        if (node == null) {
            throw new Error("IndexError: '" + world_area_id + "' not found");
        }
        return node;
    };
    Atlas.prototype.builder = function () {
        return {
            nodes: this.nodes,
        };
    };
    /**
     * batch mutations
     *
     * if the returned object is strict equal to the prev one
     * it doesn't return a new copy
     */
    Atlas.prototype.withMutations = function (mutate) {
        var prev = this.builder();
        var next = mutate(prev);
        if (prev !== next) {
            // @ts-ignore
            return this.constructor.withBuilder(next);
        }
        else {
            return this;
        }
    };
    /**
     * removes mods on all maps
     *
     * always returns a new copy
     */
    Atlas.prototype.reset = function () {
        var _this = this;
        return this.withMutations(function (_a) {
            var nodes = _a.nodes, builder = __rest(_a, ["nodes"]);
            return __assign({}, builder, { nodes: Atlas.buildLookupTable(_this.asArray().map(function (node) { return node.removeAllMods().props; })) });
        });
    };
    Atlas.prototype.addMod = function (mod, world_area_id) {
        return this.mutateNode(world_area_id, function (node) { return node.addMod(mod); });
    };
    Atlas.prototype.removeMod = function (mod, world_area_id) {
        return this.mutateNode(world_area_id, function (node) { return node.removeMod(mod); });
    };
    Atlas.prototype.mutateNode = function (world_area_id, mutate) {
        var target = this.get(world_area_id);
        var mutated = mutate(target);
        if (target === mutated) {
            return this;
        }
        else {
            return this.withMutations(function (_a) {
                var nodes = _a.nodes, builder = __rest(_a, ["nodes"]);
                return __assign({}, builder, { nodes: new Map(nodes).set(world_area_id, mutated) });
            });
        }
    };
    Atlas.prototype.applySextant = function (sextant, world_area_id) {
        var sextant_on_atlas = this.prepareSextant(sextant);
        return this.mutateNode(world_area_id, function (node) {
            return sextant_on_atlas.applyTo(node);
        });
    };
    Atlas.prototype.modsFor = function (sextant, world_area_id) {
        var sextant_on_atlas = this.prepareSextant(sextant);
        return sextant_on_atlas.modsFor(this.get(world_area_id));
    };
    Atlas.prototype.blockedMods = function (world_area_id) {
        var target = this.get(world_area_id);
        return Sextant_1.default.blockedMods(target, this.asArray());
    };
    Atlas.prototype.prepareSextant = function (sextant) {
        var clone = new Sextant_1.default(sextant.mods);
        clone.type = sextant.type;
        clone.atlas = this.asArray();
        return clone;
    };
    return Atlas;
}());
exports.default = Atlas;
