"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Stat_1 = require("../calculator/Stat");
var ts_1 = require("../util/ts");
var Mod = /** @class */ (function () {
    function Mod(props) {
        this.props = props;
    }
    Mod.build = function (props) {
        return new this(props);
    };
    Mod.prototype.isType = function (type) {
        var t = type.toUpperCase();
        return (t in Mod.GENERATION_TYPE &&
            this.props.generation_type === Mod.GENERATION_TYPE[t]);
    };
    Mod.prototype.isPrefix = function () {
        return this.isType('prefix');
    };
    Mod.prototype.isSuffix = function () {
        return this.isType('suffix');
    };
    Mod.prototype.isEnchantment = function () {
        return this.isType('enchantment');
    };
    Mod.prototype.isPremade = function () {
        return this.isType('premade');
    };
    Mod.prototype.isAffix = function () {
        return this.isPrefix() || this.isSuffix();
    };
    Mod.prototype.isMasterMod = function () {
        return this.props.domain === Mod.DOMAIN.MASTER;
    };
    Mod.prototype.implicitCandidate = function () {
        return (this.isPremade() || this.isType('vaal') || this.isType('enchantment'));
    };
    Mod.prototype.statsJoined = function () {
        var _this = this;
        return this.props.stats.map(function (stat_props, i) {
            return _this.nthStat(stat_props, i);
        });
    };
    /**
     * string identifier of the generation type
     */
    Mod.prototype.modType = function () {
        var _this = this;
        var entry = Object.entries(Mod.GENERATION_TYPE).find(function (_a) {
            var type = _a[1];
            return _this.props.generation_type === type;
        });
        if (entry === undefined) {
            return undefined;
        }
        else {
            return entry[0].toLowerCase();
        }
    };
    Mod.prototype.requiredLevel = function () {
        return Math.floor(this.props.level * 0.8);
    };
    Mod.prototype.spawnweightPropsFor = function (other) {
        var spawn_weights = this.props.spawn_weights;
        var other_tags = other.getTags();
        var match = spawn_weights.find(function (_a) {
            var tag = _a.tag;
            return other_tags.find(function (item_tag) { return tag === item_tag; }) != null;
        });
        if (match == null) {
            var default_spawnweight = spawn_weights.find(function (_a) {
                var tag = _a.tag;
                return tag === 'default';
            });
            return default_spawnweight;
        }
        else {
            return match;
        }
    };
    Mod.prototype.spawnweightFor = function (other) {
        var spawnweight = this.spawnweightPropsFor(other);
        if (spawnweight == null) {
            return 0;
        }
        else {
            return spawnweight.value;
        }
    };
    Mod.prototype.nthStat = function (stat_props, n) {
        var min_key = "stat" + (n + 1) + "_min";
        var max_key = "stat" + (n + 1) + "_max";
        if (ts_1.isKeyOf(min_key, this.props) && ts_1.isKeyOf(max_key, this.props)) {
            var stat = new Stat_1.default(stat_props, [
                +this.props[min_key],
                +this.props[max_key],
            ]);
            return stat;
        }
        throw new Error(min_key + " and " + max_key + " not found in ModProps");
    };
    Mod.DOMAIN = {
        ITEM: 1,
        FLASK: 2,
        MONSTER: 3,
        STRONGBOX: 4,
        MAP: 5,
        MASTER: 9,
        JEWEL: 10,
        ATLAS: 11,
        LEAGUESTONE: 12,
        ABYSS_JEWEL: 13,
        MAP_DEVICE: 14,
    };
    Mod.GENERATION_TYPE = {
        PREFIX: 1,
        SUFFIX: 2,
        PREMADE: 3,
        NEMESIS: 4,
        VAAL: 5,
        BLOODLINES: 6,
        TORMENT: 7,
        TEMPEST: 8,
        TALISMAN: 9,
        ENCHANTMENT: 10,
    };
    return Mod;
}());
exports.default = Mod;
