import ValueRange from './ValueRange';
import applications from './stat_applications';
// truncate after precision digits
var poe_round = function (n, precision) {
    // (Math.floor(n * 10 ** precision) / 10 ** precision
    // 1.005 => 1.00499999999999999999
    // round to prec + 1 and then truncate by removing the created digit
    return Number(n.toFixed(precision + 1).slice(0, -1));
};
var Value = /** @class */ (function () {
    function Value(range, classification, modifiers) {
        if (classification === void 0) { classification = []; }
        if (modifiers === void 0) { modifiers = []; }
        /**
         * if the value change since init
         */
        this.augmented = false;
        this.range = range instanceof ValueRange ? range : new ValueRange(range);
        this.classification = classification;
        this.modifiers = modifiers;
    }
    Object.defineProperty(Value.prototype, "value", {
        get: function () {
            return this.range.valueOf();
        },
        enumerable: true,
        configurable: true
    });
    Value.prototype.augmentWith = function (stats) {
        var _this = this;
        return new Value(this.range, this.classification, this.modifiers.concat(stats.filter(function (stat) { return _this.augmentableBy(stat); }).map(function (stat) {
            return {
                stat: stat,
                type: applications[stat.props.id].type,
            };
        })));
    };
    Value.prototype.augmentableBy = function (stat) {
        var _this = this;
        return (applications[stat.props.id] != null &&
            // empty clauses never augment anything
            applications[stat.props.id].classification.length > 0 &&
            // applications[stat.props.id].classification is in DNF
            // [a, b, [c, d]] => a || b || (c && d)
            applications[stat.props.id].classification.some(function (clause) {
                return Array.isArray(clause)
                    ? clause.every(function (term) { return _this.classification.includes(term); })
                    : _this.classification.includes(clause);
            }));
    };
    /**
     * calculates the final ValueRange from all the applied modifers
     *
     * in PoE all increase modifers get summed up to one big more modifier
     */
    Value.prototype.compute = function (precision) {
        if (precision === void 0) { precision = 0; }
        var flat = this.modifiers
            .filter(function (_a) {
            var type = _a.type;
            return type === 'flat';
        })
            .reduce(function (sum, modifier) { return sum.add(modifier.stat.values); }, ValueRange.zero);
        var increases = this.modifiers
            .filter(function (_a) {
            var type = _a.type;
            return type === 'inc';
        })
            .reduce(function (sum, modifier) { return sum.add(modifier.stat.values); }, ValueRange.zero);
        var more = this.modifiers
            .filter(function (_a) {
            var type = _a.type;
            return type === 'more';
        })
            .reduce(function (sum, modifier) { return sum.mult(modifier.stat.values.percentToFactor()); }, ValueRange.one);
        var calculated = new Value(this.range
            .add(flat)
            .mult(increases.percentToFactor())
            .mult(more)
            .map(function (n) { return poe_round(n, precision); }));
        calculated.augmented = calculated.range !== this.range;
        return calculated;
    };
    return Value;
}());
export default Value;
