export var AtlasModifier;
(function (AtlasModifier) {
    AtlasModifier["NONE"] = "";
    AtlasModifier["ELDER"] = "elder_item";
    AtlasModifier["SHAPER"] = "shaper_item";
})(AtlasModifier || (AtlasModifier = {}));
export var AtlasModifierTag;
(function (AtlasModifierTag) {
    AtlasModifierTag[AtlasModifierTag["shaper_item"] = 0] = "shaper_item";
    AtlasModifierTag[AtlasModifierTag["elder_item"] = 1] = "elder_item";
    AtlasModifierTag[AtlasModifierTag["boots_shaper"] = 2] = "boots_shaper";
    AtlasModifierTag[AtlasModifierTag["boots_elder"] = 3] = "boots_elder";
    AtlasModifierTag[AtlasModifierTag["sword_shaper"] = 4] = "sword_shaper";
    AtlasModifierTag[AtlasModifierTag["sword_elder"] = 5] = "sword_elder";
    AtlasModifierTag[AtlasModifierTag["gloves_shaper"] = 6] = "gloves_shaper";
    AtlasModifierTag[AtlasModifierTag["gloves_elder"] = 7] = "gloves_elder";
    AtlasModifierTag[AtlasModifierTag["helmet_shaper"] = 8] = "helmet_shaper";
    AtlasModifierTag[AtlasModifierTag["helmet_elder"] = 9] = "helmet_elder";
    AtlasModifierTag[AtlasModifierTag["body_armour_shaper"] = 10] = "body_armour_shaper";
    AtlasModifierTag[AtlasModifierTag["body_armour_elder"] = 11] = "body_armour_elder";
    AtlasModifierTag[AtlasModifierTag["amulet_shaper"] = 12] = "amulet_shaper";
    AtlasModifierTag[AtlasModifierTag["amulet_elder"] = 13] = "amulet_elder";
    AtlasModifierTag[AtlasModifierTag["ring_shaper"] = 14] = "ring_shaper";
    AtlasModifierTag[AtlasModifierTag["ring_elder"] = 15] = "ring_elder";
    AtlasModifierTag[AtlasModifierTag["belt_shaper"] = 16] = "belt_shaper";
    AtlasModifierTag[AtlasModifierTag["belt_elder"] = 17] = "belt_elder";
    AtlasModifierTag[AtlasModifierTag["quiver_shaper"] = 18] = "quiver_shaper";
    AtlasModifierTag[AtlasModifierTag["quiver_elder"] = 19] = "quiver_elder";
    AtlasModifierTag[AtlasModifierTag["shield_shaper"] = 20] = "shield_shaper";
    AtlasModifierTag[AtlasModifierTag["shield_elder"] = 21] = "shield_elder";
    AtlasModifierTag[AtlasModifierTag["2h_sword_shaper"] = 22] = "2h_sword_shaper";
    AtlasModifierTag[AtlasModifierTag["2h_sword_elder"] = 23] = "2h_sword_elder";
    AtlasModifierTag[AtlasModifierTag["axe_shaper"] = 24] = "axe_shaper";
    AtlasModifierTag[AtlasModifierTag["axe_elder"] = 25] = "axe_elder";
    AtlasModifierTag[AtlasModifierTag["mace_shaper"] = 26] = "mace_shaper";
    AtlasModifierTag[AtlasModifierTag["mace_elder"] = 27] = "mace_elder";
    AtlasModifierTag[AtlasModifierTag["claw_shaper"] = 28] = "claw_shaper";
    AtlasModifierTag[AtlasModifierTag["claw_elder"] = 29] = "claw_elder";
    AtlasModifierTag[AtlasModifierTag["bow_shaper"] = 30] = "bow_shaper";
    AtlasModifierTag[AtlasModifierTag["bow_elder"] = 31] = "bow_elder";
    AtlasModifierTag[AtlasModifierTag["dagger_shaper"] = 32] = "dagger_shaper";
    AtlasModifierTag[AtlasModifierTag["dagger_elder"] = 33] = "dagger_elder";
    AtlasModifierTag[AtlasModifierTag["2h_axe_shaper"] = 34] = "2h_axe_shaper";
    AtlasModifierTag[AtlasModifierTag["2h_axe_elder"] = 35] = "2h_axe_elder";
    AtlasModifierTag[AtlasModifierTag["2h_mace_shaper"] = 36] = "2h_mace_shaper";
    AtlasModifierTag[AtlasModifierTag["2h_mace_elder"] = 37] = "2h_mace_elder";
    AtlasModifierTag[AtlasModifierTag["staff_shaper"] = 38] = "staff_shaper";
    AtlasModifierTag[AtlasModifierTag["staff_elder"] = 39] = "staff_elder";
    AtlasModifierTag[AtlasModifierTag["sceptre_shaper"] = 40] = "sceptre_shaper";
    AtlasModifierTag[AtlasModifierTag["sceptre_elder"] = 41] = "sceptre_elder";
    AtlasModifierTag[AtlasModifierTag["wand_shaper"] = 42] = "wand_shaper";
    AtlasModifierTag[AtlasModifierTag["wand_elder"] = 43] = "wand_elder";
})(AtlasModifierTag || (AtlasModifierTag = {}));
export default function atlasModifier(baseitem) {
    var has_elder_tag = baseitem.tags.find(function (tag) { return tag === AtlasModifierTag[AtlasModifierTag.elder_item]; }) !== undefined;
    var has_shaper_tag = baseitem.tags.find(function (tag) { return tag === AtlasModifierTag[AtlasModifierTag.shaper_item]; }) !== undefined;
    if (has_elder_tag && has_shaper_tag) {
        throw new Error('Item can only be shaper or elder item not both.');
    }
    return fromFlags({ elder: has_elder_tag, shaper: has_shaper_tag });
}
/**
 * returns an AtlasModifier according to what flag is set
 * e.g. elder: true returns AtlasModifier.ELDER
 * throws if flags are inconsistent
 * @param param0
 */
export function fromFlags(_a) {
    var elder = _a.elder, shaper = _a.shaper;
    if (elder && shaper) {
        throw new Error('Item can only be shaper or elder item not both.');
    }
    if (elder) {
        return AtlasModifier.ELDER;
    }
    else if (shaper) {
        return AtlasModifier.SHAPER;
    }
    else {
        return AtlasModifier.NONE;
    }
}
// generates the appropriate tags for {baseitem} with {modifier}
export function tagsWithModifier(baseitem, meta_data, modifier) {
    var tags = baseitem.tags;
    var with_none = tags.filter(function (tag) {
        return !tag.endsWith('_shaper') &&
            !tag.endsWith('_elder') &&
            tag !== AtlasModifierTag[AtlasModifierTag.elder_item] &&
            tag !== AtlasModifierTag[AtlasModifierTag.shaper_item];
    });
    switch (modifier) {
        case AtlasModifier.NONE:
            return with_none;
        case AtlasModifier.ELDER:
            return with_none.concat(AtlasModifierTag[AtlasModifierTag.elder_item], AtlasModifierTag[elderTag(meta_data)]);
        case AtlasModifier.SHAPER:
            return with_none.concat(AtlasModifierTag[AtlasModifierTag.shaper_item], AtlasModifierTag[shaperTag(meta_data)]);
    }
}
// {baseitem} specific elder tag
export function elderTag(meta_data) {
    try {
        return suffixedTag('elder', meta_data);
    }
    catch (err) {
        throw new Error("this item cannot have the elder tag (" + err.message + ")");
    }
}
// {baseitem} specific shaperTag tag
export function shaperTag(meta_data) {
    try {
        return suffixedTag('shaper', meta_data);
    }
    catch (err) {
        throw new Error("this item cannot have the shaper tag (" + err.message + ")");
    }
}
function suffixedTag(suffix, meta_data) {
    var tag_prefix = tagIdentifier(meta_data);
    var tag = AtlasModifierTag[tag_prefix + "_" + suffix];
    if (tag !== undefined) {
        return tag;
    }
    else {
        throw new Error(tag_prefix + " not set in Tag with '" + suffix + "' as suffix");
    }
}
function tagIdentifier(meta_data) {
    if (meta_data.isA('AbstractTwoHandAxe')) {
        return '2h_axe';
    }
    else if (meta_data.isA('AbstractTwoHandMace')) {
        return '2h_mace';
    }
    else if (meta_data.isA('AbstractTwoHandSword')) {
        return '2h_sword';
    }
    else if (meta_data.isA('AbstractAmulet')) {
        return 'amulet';
    }
    else if (meta_data.isA('AbstractOneHandAxe')) {
        return 'axe';
    }
    else if (meta_data.isA('AbstractBelt')) {
        return 'belt';
    }
    else if (meta_data.isA('AbstractBodyArmour')) {
        return 'body_armour';
    }
    else if (meta_data.isA('AbstractBoots')) {
        return 'boots';
    }
    else if (meta_data.isA('AbstractBow')) {
        return 'bow';
    }
    else if (meta_data.isA('AbstractClaw')) {
        return 'claw';
    }
    else if (meta_data.isA('AbstractDagger')) {
        return 'dagger';
    }
    else if (meta_data.isA('AbstractGloves')) {
        return 'gloves';
    }
    else if (meta_data.isA('AbstractHelmet')) {
        return 'helmet';
    }
    else if (meta_data.isA('AbstractOneHandMace')) {
        return 'mace';
    }
    else if (meta_data.isA('AbstractQuiver')) {
        return 'quiver';
    }
    else if (meta_data.isA('AbstractRing')) {
        return 'ring';
    }
    else if (meta_data.isA('AbstractSceptre')) {
        return 'sceptre';
    }
    else if (meta_data.isA('AbstractShield')) {
        return 'shield';
    }
    else if (meta_data.isA('AbstractStaff')) {
        return 'staff';
    }
    else if (meta_data.isA('AbstractOneHandSword')) {
        return 'sword';
    }
    else if (meta_data.isA('AbstractWand')) {
        return 'wand';
    }
    throw new Error();
}
