var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { Mod, metaMods as META_MODS } from '../../../mods';
import ImmutableContainer from '../../ImmutableContainer';
/**
 * the explicits of an item
 */
var ItemAffixes = /** @class */ (function (_super) {
    __extends(ItemAffixes, _super);
    function ItemAffixes(item, mods) {
        var _this = _super.call(this, mods) || this;
        _this.item = item;
        return _this;
    }
    ItemAffixes.withBuilder = function (builder) {
        return new ItemAffixes(builder.item, builder.mods);
    };
    ItemAffixes.prototype.builder = function () {
        return {
            item: this.item,
            mods: this.mods,
        };
    };
    /**
     * @override
     */
    ItemAffixes.prototype.maxModsOfType = function (mod) {
        if (mod.isPrefix()) {
            return this.maxPrefixes();
        }
        else if (mod.isSuffix()) {
            return this.maxSuffixes();
        }
        else {
            return 0;
        }
    };
    /**
     *  checks if the domains are equiv
     */
    ItemAffixes.prototype.inDomainOf = function (mod_domain) {
        switch (mod_domain) {
            case Mod.DOMAIN.MASTER:
                return this.inDomainOf(Mod.DOMAIN.ITEM);
            default:
                return mod_domain === this.modDomainEquiv();
        }
    };
    ItemAffixes.prototype.level = function () {
        return this.item.props.item_level;
    };
    ItemAffixes.prototype.lockedPrefixes = function () {
        return this.indexOfModWithId(META_MODS.LOCKED_PREFIXES) !== -1;
    };
    ItemAffixes.prototype.lockedSuffixes = function () {
        return this.indexOfModWithId(META_MODS.LOCKED_SUFFIXES) !== -1;
    };
    ItemAffixes.prototype.getPrefixes = function () {
        return this.mods.filter(function (mod) { return mod.isPrefix(); });
    };
    ItemAffixes.prototype.getSuffixes = function () {
        return this.mods.filter(function (mod) { return mod.isSuffix(); });
    };
    /**
     * maximum number of prefixes
     */
    ItemAffixes.prototype.maxPrefixes = function () {
        if (this.item.rarity.isNormal()) {
            return 0;
        }
        else if (this.item.rarity.isMagic()) {
            return 1;
        }
        else if (this.item.rarity.isRare()) {
            if (this.item.meta_data.isA('AbstractJewel')) {
                return 2;
            }
            else if (this.item.meta_data.isA('AbstractAbyssJewel')) {
                return 2;
            }
            return 3;
        }
        else if (this.item.rarity.isUnique()) {
            return Number.POSITIVE_INFINITY;
        }
        else {
            throw new Error('rarity not recognized');
        }
    };
    ItemAffixes.prototype.maxSuffixes = function () {
        return this.maxPrefixes();
    };
    ItemAffixes.prototype.modDomainEquiv = function () {
        if (this.item.meta_data.isA('AbstractJewel')) {
            return Mod.DOMAIN.JEWEL;
        }
        if (this.item.meta_data.isA('AbstractFlask')) {
            return Mod.DOMAIN.FLASK;
        }
        if (this.item.meta_data.isA('AbstractMap')) {
            return Mod.DOMAIN.MAP;
        }
        if (this.item.meta_data.isA('AbstractAbyssJewel')) {
            return Mod.DOMAIN.ABYSS_JEWEL;
        }
        return Mod.DOMAIN.ITEM;
    };
    return ItemAffixes;
}(ImmutableContainer));
export default ItemAffixes;
