import Component from '../../Component';
import Item from '../../Item';
import { SocketOptions, ColorBatch } from './options';
import { SocketColor, SocketGroup, SocketId, Socket } from './types';
export interface Sockets<T> {
    /**
     * @deprecated use maxSockets() instead
     * @returns the maximum amount of sockets
     */
    max(): number;
    /**
     * @returns the maximum amount of sockets
     */
    maxSockets(): number;
    /**
     * @returns current amount of sockets
     */
    count(): number;
    /**
     * @returns the biggest number of linked sockets
     */
    maxLinks(): number;
    /**
     * checks if {a} and {b} are linked
     * @param a the socket to check
     * @param b the socket for which to check if it is linked to a
     */
    isLinked(a: SocketId, b: SocketId): boolean;
    /**
     * links each pair of
     * @param a
     * @param b
     */
    link(...sockets: SocketId[]): T;
    /**
     * sets the number of sockets to n
     * @param n
     * @param options set {keep_links} to true if it should not reroll links
     */
    socket(n: number, options?: Partial<SocketOptions>): T;
    colors(): SocketColor[];
    color(color: SocketColor, ...sockets: SocketId[]): T;
    colorBatch(...batch: ColorBatch[]): T;
    mapColors(fn: (socket: Socket, index: SocketId, item: T) => Socket): T;
    toJson(): Array<Socket & {
        group: number;
    }>;
}
export interface Builder {
    sockets: Socket[];
    groups: SocketGroup[];
}
/**
 * item component for sockets
 */
export default class ItemSockets implements Sockets<Item>, Component<Item, Builder> {
    parent: Item;
    groups: SocketGroup[];
    sockets: Socket[];
    constructor(item: Item, builder: Builder);
    builder(): {
        groups: number[][];
        sockets: Socket[];
    };
    /**
     * @deprecated
     */
    max(): number;
    /**
     * calculates the maximum amount of sockets possible on this item
     *
     * TODO: what about Corroded Blades or other similar 1x4 Items. Confirm
     * that they also only can have max 4 sockets like Rods or act like small_Staff
     */
    maxSockets(): number;
    count(): number;
    maxLinks(): number;
    isLinked(a: SocketId, b: SocketId): boolean;
    /**
     * links the specified socket indices
     * @param pairs consider args as pairs so be sure to have even argument length
     */
    link(...pairs: SocketId[]): Item;
    /**
     * set the sockets to {n}
     *
     * set force to true to ignore socket limit
     * set keep_links to false if the new sockets should all be unlinked
     * newSocket is a callback which is called for every new socket. By default
     * a white socket is created.
     *
     * @param n
     * @param options
     */
    socket(n: number, options?: Partial<SocketOptions>): Item;
    colors(): SocketColor[];
    color(color: SocketColor, ...indices: SocketId[]): Item;
    colorBatch(...batches: ColorBatch[]): Item;
    /**
     *
     * @param fn if fn returns a referentially equal Socket no new item is returned
     */
    mapColors(fn: (socket: Socket, index: number, item: Item) => Socket): Item;
    any(): boolean;
    /**
     * a string representation of the sockets, links and colors
     *
     * each char represents a color shorthand, dashes means the adjacent sockets
     * are linked, whitespace instead means no link
     */
    toString(): string;
    /**
     * serialized format which includes group information
     */
    toJson(): Array<Socket & {
        group: number;
    }>;
    private groupOf;
    private maxByMetaData;
    private maxByLevel;
    private maxByDimensions;
    private maxOverride;
    /**
     * true if some batch tries to color an index that is out of bounds
     * @param batches
     */
    private batchesHaveNonExistingSockets;
}
