import Container from '../containers/Container';
import Mod from '../mods/Mod';
import { Flags } from '../util/Flags';
export interface GeneratorDetails<M extends Mod> {
    mod: M;
    applicable?: Flags;
    spawnable?: Flags;
    spawnweight?: number;
}
export interface ModApplicableFlags extends Flags {
    domain_full: boolean;
    already_present: boolean;
    wrong_domain: boolean;
    lower_ilvl: boolean;
    above_lld_level: boolean;
}
export declare type ModApplicableFlag = keyof ModApplicableFlags;
/**
 * @abstract
 */
export default abstract class Generator<M extends Mod, C extends Container<any>> {
    mods: M[];
    constructor(mods: M[]);
    abstract applyTo(container: C): C;
    abstract modsFor(container: C, whitelist: string[]): Array<GeneratorDetails<M>>;
    abstract applicableTo(container: C): Flags;
    /**
     * returns a copy of #mods
     *
     * we can stick with a shallow copy since Mod are supposed to be immutable
     */
    getAvailableMods(): M[];
    isApplicableTo(container: C, whitelist?: string[]): boolean;
    isModApplicableTo(mod: M, container: C): ModApplicableFlags;
}
