var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import MasterBench from '../helpers/MasterBench';
import Generator from './Generator';
import { Alchemy, EnchantmentBench, Essence, Vaal } from './item_orbs';
/**
 * Masterbench/Currency hybrid
 */
var ItemShowcase = /** @class */ (function (_super) {
    __extends(ItemShowcase, _super);
    function ItemShowcase(props, options, essences) {
        var _this = this;
        var enchantment = EnchantmentBench.build(props);
        var master = MasterBench.build(options);
        var explicits = Alchemy.build(props);
        var vaal = Vaal.build(props);
        var mods = enchantment.mods.concat(explicits.mods, vaal.mods, master.getAvailableMods());
        _this = _super.call(this, mods) || this;
        _this.enchantment = enchantment;
        _this.master = master;
        _this.explicits = explicits;
        _this.vaal = vaal;
        _this.essences = essences;
        return _this;
    }
    /**
     * only abstract showcase, not for actual usage
     */
    ItemShowcase.prototype.applyTo = function (item) {
        return item;
    };
    /**
     * not applicable to anything
     * @param item
     */
    ItemShowcase.prototype.applicableTo = function (item) {
        return {
            not_applicable: true,
        };
    };
    /**
     * greps mod::applicableTo and (if implemented) mod::spawnableOn
     * if we have all the space for mods we need
     *
     * @returns master-, enchantment-, vaal-, explicit-, (top tier) essence-mods
     */
    ItemShowcase.prototype.modsFor = function (item, whitelist) {
        if (whitelist === void 0) { whitelist = []; }
        var details = this.master.modsFor(item, whitelist).concat(this.enchantment.modsFor(item, whitelist), this.explicits.modsFor(item, whitelist), this.vaal.modsFor(item, whitelist), this.essences
            .map(function (props) {
            var essence = new Essence(props, []);
            if (essence.isTopTier()) {
                return essence.chooseMod(item);
            }
            else {
                return undefined;
            }
        })
            .filter(function (mod) {
            return mod !== undefined;
        })
            .map(function (mod) { return ({ mod: mod }); }));
        // flow cant merge object types
        // { mod: OneMod, prop: number } | { mod: AnotherMod, prop: number }
        // will not become { mod: OneMod | AnotherMod, prop: number }
        // and for some reason ['a', 'b'] cant be cast to Array<number | string>
        return details;
    };
    return ItemShowcase;
}(Generator));
export default ItemShowcase;
