import Item from '../containers/item/Item';
import { CraftingBenchOptionsProps } from '../schema';
import { Flags } from '../util/Flags';
import { Mod } from '../mods';
import Generator, { ModApplicableFlags as BaseModApplicableFlags, GeneratorDetails } from './Generator';
export interface ModApplicableFlags extends BaseModApplicableFlags {
    no_multimod: boolean;
}
export declare type ModApplicableFlag = keyof BaseModApplicableFlags;
export interface ApplicableFlags extends Flags {
    wrong_itemclass: boolean;
}
export declare type ApplicableFlag = keyof ApplicableFlags;
export default class MasterBenchOption extends Generator<Mod, Item> {
    static build(option: CraftingBenchOptionsProps): MasterBenchOption;
    readonly props: CraftingBenchOptionsProps;
    constructor(option: CraftingBenchOptionsProps);
    readonly mod: Mod | undefined;
    /**
     * applies a chosen craftingbenchoption
     *
     * cant overload extended method. so we have to set the chosen option before
     */
    applyTo(item: Item): Item;
    /**
     * Can accept only certain itemclasses
     */
    applicableTo(item: Item): ApplicableFlags;
    /**
     * greps mod::applicableTo
     */
    modsFor(item: Item, whitelist?: string[]): Array<GeneratorDetails<Mod>>;
    /**
     * checks if the given mod is applicable to the item
     *
     * remember that this doesn't check if the passed mod is the mod of this option
     */
    isModApplicableTo(mod: Mod, item: Item): ModApplicableFlags;
}
