var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import { anySet } from '../util/Flags';
import { filterUndefined } from '../util/ts';
import { metaMods as META_MODs, Mod } from '../mods';
import Generator from './Generator';
var MasterBenchOption = /** @class */ (function (_super) {
    __extends(MasterBenchOption, _super);
    function MasterBenchOption(option) {
        var _this = this;
        if (option.mod != null) {
            _this = _super.call(this, [new Mod(option.mod)]) || this;
        }
        else {
            _this = _super.call(this, []) || this;
        }
        _this.props = option;
        return _this;
    }
    MasterBenchOption.build = function (option) {
        return new MasterBenchOption(option);
    };
    Object.defineProperty(MasterBenchOption.prototype, "mod", {
        get: function () {
            return this.mods[0];
        },
        enumerable: true,
        configurable: true
    });
    /**
     * applies a chosen craftingbenchoption
     *
     * cant overload extended method. so we have to set the chosen option before
     */
    MasterBenchOption.prototype.applyTo = function (item) {
        if (this.isApplicableTo(item)) {
            var mod = this.mod;
            /**
             * TODO customactions for no mod
             */
            if (mod != null) {
                // white gets upgraded to blue
                var crafted_item = item.rarity.isNormal()
                    ? item.rarity.set('magic')
                    : item;
                if (this.isModApplicableTo(mod, crafted_item)) {
                    return crafted_item.addMod(mod);
                }
            }
            else {
                throw new Error('customactions are not implemented yet');
            }
        }
        // nothing changed
        return item;
    };
    /**
     * Can accept only certain itemclasses
     */
    MasterBenchOption.prototype.applicableTo = function (item) {
        var applicable_flags = {
            wrong_itemclass: false,
        };
        var item_classes = this.props.item_classes;
        applicable_flags.wrong_itemclass =
            // no item classes means every item is welcome
            item_classes.length > 0 &&
                item_classes.find(function (item_class) { return item_class === item.baseitem.item_class; }) === undefined;
        return applicable_flags;
    };
    /**
     * greps mod::applicableTo
     */
    MasterBenchOption.prototype.modsFor = function (item, whitelist) {
        var _this = this;
        if (whitelist === void 0) { whitelist = []; }
        // TODO look into why we simulate another rarity why is a MasterMod not
        // applicable to white items?
        // simulate blue if white
        var simulated_item = item.rarity.isNormal()
            ? item.rarity.set('magic')
            : item;
        return this.getAvailableMods()
            .map(function (mod) {
            var applicable_flags = __assign({}, _this.isModApplicableTo(mod, simulated_item), _this.applicableTo(simulated_item));
            if (anySet(applicable_flags, whitelist)) {
                return null;
            }
            else {
                return {
                    mod: mod,
                    applicable: applicable_flags,
                };
            }
        })
            .filter(filterUndefined);
    };
    /**
     * checks if the given mod is applicable to the item
     *
     * remember that this doesn't check if the passed mod is the mod of this option
     */
    MasterBenchOption.prototype.isModApplicableTo = function (mod, item) {
        var applicable_flags = __assign({}, _super.prototype.isModApplicableTo.call(this, mod, item), { no_multimod: false });
        // grep MasterMods and set failure if we cant multimod
        var master_mods = item.mods.filter(function (other) { return other.isMasterMod(); });
        var has_no_multi_mod = master_mods.find(function (other) { return other.props.id === META_MODs.MULTIMOD; }) ===
            undefined;
        if (master_mods.length > 0 && has_no_multi_mod) {
            applicable_flags.no_multimod = true;
        }
        return applicable_flags;
    };
    return MasterBenchOption;
}(Generator));
export default MasterBenchOption;
