import Item from '../../containers/item/Item';
import { CraftingBenchOptionsProps } from '../../schema';
import { Flags } from '../../util/Flags';
import { Mod } from '../../mods';
import Generator, { ModApplicableFlags as BaseModApplicableFlags, GeneratorDetails } from '../Generator';
export interface ModApplicableFlags extends BaseModApplicableFlags {
    no_multimod: boolean;
}
export declare type ModApplicableFlag = keyof BaseModApplicableFlags;
export interface ApplicableFlags extends Flags {
    wrong_itemclass: boolean;
    /**
     * only relevant to "X Sockets"
     */
    socket_limit_exceeded: boolean;
    /**
     * only relevant to "X Sockets"
     */
    sockets_unmodified: boolean;
    /**
     * only relevant to "X Links"
     */
    not_enough_sockets: boolean;
    /**
     * only relevant to "X Links"
     */
    links_unmodified: boolean;
    /**
     * only relevant to "Remove crafted mot"
     */
    no_crafted_mod: boolean;
    /**
     * only relevant to options that add mods
     */
    mod_not_applicable: boolean;
}
export declare type ApplicableFlag = keyof ApplicableFlags;
export default class MasterBenchOption extends Generator<Mod, Item> {
    static build(option: CraftingBenchOptionsProps): MasterBenchOption;
    readonly props: CraftingBenchOptionsProps;
    constructor(option: CraftingBenchOptionsProps);
    readonly mod: Mod | undefined;
    /**
     * applies a chosen craftingbenchoption
     *
     * cant overload extended method. so we have to set the chosen option before
     */
    applyTo(item: Item): Item;
    /**
     * Can accept only certain itemclasses and in case of vorici options
     * the option actually has to "make sense" (e.g. not 6link of 4socket)
     */
    applicableTo(item: Item): ApplicableFlags;
    /**
     * greps mod::applicableTo
     */
    modsFor(item: Item, whitelist?: string[]): Array<GeneratorDetails<Mod>>;
    /**
     * checks if the given mod is applicable to the item
     *
     * remember that this doesn't check if the passed mod is the mod of this option
     */
    isModApplicableTo(mod: Mod, item: Item): ModApplicableFlags;
    isRemoveCraftedModOption(): boolean;
    private removeCraftedMod;
    private applyRandomZanaMod;
    private applySelfTo;
    private applyModTo;
    private applySocketChangeTo;
    private applyLinkChangeTo;
    private applySocketColorChangeTo;
}
