var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import { anySet } from '../../util/Flags';
import { filterUndefined } from '../../util/ts';
import { metaMods as META_MODs, Mod } from '../../mods';
import Generator from '../Generator';
import * as actions from './custom_actions';
var MasterBenchOption = /** @class */ (function (_super) {
    __extends(MasterBenchOption, _super);
    function MasterBenchOption(option) {
        var _this = this;
        if (option.mod != null) {
            _this = _super.call(this, [new Mod(option.mod)]) || this;
        }
        else {
            _this = _super.call(this, []) || this;
        }
        _this.props = option;
        return _this;
    }
    MasterBenchOption.build = function (option) {
        return new MasterBenchOption(option);
    };
    Object.defineProperty(MasterBenchOption.prototype, "mod", {
        get: function () {
            return this.mods[0];
        },
        enumerable: true,
        configurable: true
    });
    /**
     * applies a chosen craftingbenchoption
     *
     * cant overload extended method. so we have to set the chosen option before
     */
    MasterBenchOption.prototype.applyTo = function (item) {
        if (this.isApplicableTo(item)) {
            var custom_action = this.props.crafting_bench_custom_action;
            switch (custom_action) {
                case actions.CustomAction.RemoveCraftedMod:
                    return this.removeCraftedMod(item);
                case actions.CustomAction.RandomZanaMod:
                    return this.applyRandomZanaMod(item);
                case actions.CustomAction.ApplySelf:
                    return this.applySelfTo(item);
                default:
                    throw new actions.UnrecognizedCustomAction(custom_action);
            }
        }
        // nothing changed
        return item;
    };
    /**
     * Can accept only certain itemclasses and in case of vorici options
     * the option actually has to "make sense" (e.g. not 6link of 4socket)
     */
    MasterBenchOption.prototype.applicableTo = function (item) {
        var _a = this.props, item_classes = _a.item_classes, sockets = _a.sockets, links = _a.links, socket_colours = _a.socket_colours;
        var wrong_itemclass = 
        // no item classes means every item is welcome
        item_classes.length > 0 &&
            item_classes.find(function (item_class) { return item_class === item.baseitem.item_class; }) === undefined;
        var socket_limit_exceeded = sockets > 0 && sockets > item.sockets.maxSockets();
        var sockets_unmodified = sockets > 0 && sockets === item.sockets.count();
        var not_enough_sockets = 
        // more linkes than sockets
        (links > 0 && links > item.sockets.count()) ||
            // more colors thant sockets
            (socket_colours.length > 0 &&
                socket_colours.length >= item.sockets.count());
        var links_unmodified = links > 0 && item.sockets.isLinked(0, links - 1);
        var no_crafted_mod = this.isRemoveCraftedModOption() &&
            !item.mods.some(function (mod) { return mod.isMasterMod(); });
        var designated_mod = this.mod;
        var mod_not_applicable = designated_mod != null &&
            anySet(this.isModApplicableTo(designated_mod, item));
        return {
            wrong_itemclass: wrong_itemclass,
            socket_limit_exceeded: socket_limit_exceeded,
            sockets_unmodified: sockets_unmodified,
            not_enough_sockets: not_enough_sockets,
            links_unmodified: links_unmodified,
            no_crafted_mod: no_crafted_mod,
            mod_not_applicable: mod_not_applicable,
        };
    };
    /**
     * greps mod::applicableTo
     */
    MasterBenchOption.prototype.modsFor = function (item, whitelist) {
        var _this = this;
        if (whitelist === void 0) { whitelist = []; }
        // TODO look into why we simulate another rarity why is a MasterMod not
        // applicable to white items?
        // simulate blue if white
        var simulated_item = item.rarity.isNormal()
            ? item.rarity.set('magic')
            : item;
        return this.getAvailableMods()
            .map(function (mod) {
            var applicable_flags = __assign({}, _this.isModApplicableTo(mod, simulated_item), _this.applicableTo(simulated_item));
            if (anySet(applicable_flags, whitelist)) {
                return null;
            }
            else {
                return {
                    mod: mod,
                    applicable: applicable_flags,
                };
            }
        })
            .filter(filterUndefined);
    };
    /**
     * checks if the given mod is applicable to the item
     *
     * remember that this doesn't check if the passed mod is the mod of this option
     */
    MasterBenchOption.prototype.isModApplicableTo = function (mod, item) {
        var simulated_item = item.rarity.isNormal()
            ? item.rarity.set('magic')
            : item;
        var applicable_flags = __assign({}, _super.prototype.isModApplicableTo.call(this, mod, simulated_item), { no_multimod: false });
        // grep MasterMods and set failure if we cant multimod
        var master_mods = item.mods.filter(function (other) { return other.isMasterMod(); });
        var has_no_multi_mod = master_mods.find(function (other) { return other.props.id === META_MODs.MULTIMOD; }) ===
            undefined;
        if (master_mods.length > 0 && has_no_multi_mod) {
            applicable_flags.no_multimod = true;
        }
        return applicable_flags;
    };
    MasterBenchOption.prototype.isRemoveCraftedModOption = function () {
        return (this.props.crafting_bench_custom_action ===
            actions.CustomAction.RemoveCraftedMod);
    };
    MasterBenchOption.prototype.removeCraftedMod = function (item) {
        return actions.removeCraftedMods(item);
    };
    MasterBenchOption.prototype.applyRandomZanaMod = function (item) {
        throw new Error('not implemented');
    };
    MasterBenchOption.prototype.applySelfTo = function (item) {
        var _a = this, mod = _a.mod, props = _a.props;
        if (mod != null) {
            return this.applyModTo(item, mod);
        }
        else if (props.sockets > 0) {
            return this.applySocketChangeTo(item);
        }
        else if (props.links > 0) {
            return this.applyLinkChangeTo(item);
        }
        else if (props.socket_colours.length > 0) {
            return this.applySocketColorChangeTo(item);
        }
        else {
            throw new Error("Don't know what to do with this option.");
        }
    };
    MasterBenchOption.prototype.applyModTo = function (item, mod) {
        // white gets upgraded to blue
        var crafted_item = item.rarity.isNormal()
            ? item.rarity.set('magic')
            : item;
        return crafted_item.addMod(mod);
    };
    MasterBenchOption.prototype.applySocketChangeTo = function (item) {
        return actions.changeSockets(item, this.props.sockets);
    };
    MasterBenchOption.prototype.applyLinkChangeTo = function (item) {
        return actions.changeLinks(item, this.props.links);
    };
    MasterBenchOption.prototype.applySocketColorChangeTo = function (item) {
        return actions.changeColors(item, this.props.socket_colours);
    };
    return MasterBenchOption;
}(Generator));
export default MasterBenchOption;
