import { BaseError } from 'make-error';
import { Item } from '../../containers';
export declare enum CustomAction {
    RemoveCraftedMod = 1,
    /**
     * i.e. Fortune favours the brav
     */
    RandomZanaMod = 2,
    /**
     * i.e. apply the thing that is defined in its props
     * e.g. a mod, sockets, colors, links
     */
    ApplySelf = 3
}
/**
 * {CraftingBenchOptionsProps.crafting_bench_custom_action} is of type
 * number but at the moment there are only a finite number of actions.
 * In the future more actions might be added. If we encounter such an action
 * this error is thrown.
 */
export declare class UnrecognizedCustomAction extends BaseError {
    constructor(action: number);
}
export declare class InvalidColorConfiguration extends BaseError {
    constructor(char: string);
}
/**
 * changes the number of sockets to {n}
 * new sockets are generated with {rollRequirementBiasedStable}
 * @param item
 * @param n
 */
export declare function changeSockets(item: Item, n: number): Item;
/**
 * links the first {n} sockets of the item
 * if {n} is bigger than the current number of sockets all sockets are linked
 * @param item
 * @param n
 */
export declare function changeLinks(item: Item, n: number): Item;
/**
 * changes the colors of the item according to the configuration
 * @param item
 * @param configuration - a string consisting of R|G|B e.g. 'RRR' for 3 red
 *                        The configuration is case sensitive
 */
export declare function changeColors(item: Item, configuration: string): Item;
/**
 * removes all crafted mods, if not crafted mods are present the item itself is returned
 * @param item
 */
export declare function removeCraftedMods(item: Item): Item;
