var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { BaseError } from 'make-error';
import { SocketColor, rollRequirementBiasedStable, } from '../../containers/item/components/Sockets';
export var CustomAction;
(function (CustomAction) {
    CustomAction[CustomAction["RemoveCraftedMod"] = 1] = "RemoveCraftedMod";
    /**
     * i.e. Fortune favours the brav
     */
    CustomAction[CustomAction["RandomZanaMod"] = 2] = "RandomZanaMod";
    /**
     * i.e. apply the thing that is defined in its props
     * e.g. a mod, sockets, colors, links
     */
    CustomAction[CustomAction["ApplySelf"] = 3] = "ApplySelf";
})(CustomAction || (CustomAction = {}));
/**
 * {CraftingBenchOptionsProps.crafting_bench_custom_action} is of type
 * number but at the moment there are only a finite number of actions.
 * In the future more actions might be added. If we encounter such an action
 * this error is thrown.
 */
var UnrecognizedCustomAction = /** @class */ (function (_super) {
    __extends(UnrecognizedCustomAction, _super);
    function UnrecognizedCustomAction(action) {
        return _super.call(this, "This option has an unsupported custom action '" + action + "'") || this;
    }
    return UnrecognizedCustomAction;
}(BaseError));
export { UnrecognizedCustomAction };
var InvalidColorConfiguration = /** @class */ (function (_super) {
    __extends(InvalidColorConfiguration, _super);
    function InvalidColorConfiguration(char) {
        return _super.call(this, "Invalid character '" + char + "' in color configuration") || this;
    }
    return InvalidColorConfiguration;
}(BaseError));
export { InvalidColorConfiguration };
/**
 * changes the number of sockets to {n}
 * new sockets are generated with {rollRequirementBiasedStable}
 * @param item
 * @param n
 */
export function changeSockets(item, n) {
    return item.sockets.socket(n, {
        keep_links: true,
        newSocket: rollRequirementBiasedStable,
    });
}
/**
 * links the first {n} sockets of the item
 * if {n} is bigger than the current number of sockets all sockets are linked
 * @param item
 * @param n
 */
export function changeLinks(item, n) {
    var link_size = Math.min(item.sockets.count(), n);
    return item.sockets.link(0, link_size - 1);
}
/**
 * changes the colors of the item according to the configuration
 * @param item
 * @param configuration - a string consisting of R|G|B e.g. 'RRR' for 3 red
 *                        The configuration is case sensitive
 */
export function changeColors(item, configuration) {
    var colors = parseColorConfiguration(configuration);
    return item.sockets.mapColors(function (socket, index) {
        var color = colors[index];
        if (color === undefined) {
            return rollRequirementBiasedStable(item);
        }
        else {
            return { color: color };
        }
    });
}
/**
 * removes all crafted mods, if not crafted mods are present the item itself is returned
 * @param item
 */
export function removeCraftedMods(item) {
    return item.mods
        .filter(function (mod) { return mod.isMasterMod(); })
        .reduce(function (acc, mod) { return acc.removeMod(mod); }, item);
}
function parseColorConfiguration(configuration) {
    return configuration.split('').map(function (char) {
        switch (char) {
            case 'R':
                return SocketColor.red;
            case 'B':
                return SocketColor.blue;
            case 'G':
                return SocketColor.green;
            default:
                throw new InvalidColorConfiguration(char);
        }
    });
}
