import Container from '../containers/Container';
import { ModProps } from '../schema';
import { Mod } from '../mods';
import { Flags } from '../util/Flags';
import Generator, { GeneratorDetails } from './Generator';
export interface SpawnableFlags extends Flags {
    no_matching_tags: boolean;
    spawnweight_zero: boolean;
}
export declare type SpawnableFlag = keyof SpawnableFlags;
/**
 * @abstract
 * a Generator that randomly rolls one of its mods
 * ignores mods that have no spawnweight or 0 spawnweight for every tag
 */
export default abstract class Orb<C extends Container<any>> extends Generator<Mod, C> {
    static modFilter(mod: ModProps): boolean;
    static buildMods(mods: ModProps[]): Mod[];
    chooseMod(container: C): Mod | undefined;
    /**
     * adds a mod from chooseMod ignoring if it's applicable
     * @param {Item} item
     */
    rollMod(container: C): C;
    isModSpawnableOn(mod: Mod, container: C): SpawnableFlags;
    spawnweightFor(mod: Mod, container: C): number | undefined;
    modsFor(container: C, whitelist?: string[]): Array<GeneratorDetails<Mod>>;
}
