import { BaseError } from 'make-error';
import AtlasNode from '../containers/AtlasNode';
import Mod from '../mods/Mod';
import { ModProps } from '../schema';
import { Flags } from '../util/Flags';
import Orb, { SpawnableFlags as BaseSpawnableFlags } from './Orb';
export interface SpawnableFlags extends BaseSpawnableFlags {
    no_adjacents_with_spawnweight: boolean;
}
export declare type SpawnableFlag = keyof SpawnableFlags;
export interface ApplicableFlags extends Flags {
    wrong_tier_group: boolean;
}
export declare type ApplicableFlag = keyof ApplicableFlags;
export declare class ContextUndefined extends BaseError {
    constructor(context: string);
}
export declare class CorruptedState extends BaseError {
    constructor(message: string);
}
export declare enum Type {
    apprentice = 1,
    journeyman = 2,
    master = 3
}
export default class Sextant extends Orb<AtlasNode> {
    static type: typeof Type;
    static modFilter(mod: ModProps): boolean;
    static build(mods: ModProps[]): Sextant;
    /**
     * creates a list of mod that are either
     * a) already applied to this node because they are directly in range
     * b) already applied to adjacent nodes and would create duplicates if applied
     *    to this node
     * in other words: consider every mod from maps within a sextant distance of 2
     * @param {AtlasNode} node
     * @param {AtlasNode[]} atlas
     */
    static blockedMods(target: AtlasNode, atlas: AtlasNode[]): Mod[];
    atlas: AtlasNode[] | undefined;
    type: Type;
    applyTo(node: AtlasNode): AtlasNode;
    applicableTo(node: AtlasNode): ApplicableFlags;
    findAdjacentWithSpawnweight(mod: Mod, node: AtlasNode): AtlasNode | undefined;
    isModSpawnableOn(mod: Mod, node: AtlasNode): SpawnableFlags;
    modsFor(node: AtlasNode, whitelist?: string[]): {
        applicable?: Flags | undefined;
        mod: Mod;
        spawnable: Flags;
        spawnweight: number;
    }[];
}
