var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
import { BaseError } from 'make-error';
import Mod from '../mods/Mod';
import { anySet } from '../util/Flags';
import Orb from './Orb';
var ContextUndefined = /** @class */ (function (_super) {
    __extends(ContextUndefined, _super);
    function ContextUndefined(context) {
        return _super.call(this, "context not set, set " + context) || this;
    }
    return ContextUndefined;
}(BaseError));
export { ContextUndefined };
var CorruptedState = /** @class */ (function (_super) {
    __extends(CorruptedState, _super);
    function CorruptedState(message) {
        return _super.call(this, "corrupted state: " + message) || this;
    }
    return CorruptedState;
}(BaseError));
export { CorruptedState };
export var Type;
(function (Type) {
    Type[Type["apprentice"] = 1] = "apprentice";
    Type[Type["journeyman"] = 2] = "journeyman";
    Type[Type["master"] = 3] = "master";
})(Type || (Type = {}));
var Sextant = /** @class */ (function (_super) {
    __extends(Sextant, _super);
    function Sextant() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.type = Type.master;
        return _this;
    }
    Sextant.modFilter = function (mod) {
        return _super.modFilter.call(this, mod) && mod.domain === Mod.DOMAIN.ATLAS;
    };
    Sextant.build = function (mods) {
        return new Sextant(this.buildMods(mods));
    };
    /**
     * creates a list of mod that are either
     * a) already applied to this node because they are directly in range
     * b) already applied to adjacent nodes and would create duplicates if applied
     *    to this node
     * in other words: consider every mod from maps within a sextant distance of 2
     * @param {AtlasNode} node
     * @param {AtlasNode[]} atlas
     */
    Sextant.blockedMods = function (target, atlas) {
        return target
            .inSextantRange(atlas, 1)
            .reduce(function (mods, secondary) { return mods.concat(secondary.getAllMods(atlas)); }, []);
    };
    Sextant.prototype.applyTo = function (node) {
        if (!anySet(this.applicableTo(node))) {
            var rolled = _super.prototype.rollMod.call(this, node.removeAllMods());
            if (rolled !== node) {
                // something change
                // reset the context to signal that it has to be mutated
                this.atlas = undefined;
            }
            return rolled;
        }
        else {
            return node;
        }
    };
    // applicable to any node for now
    Sextant.prototype.applicableTo = function (node) {
        var _a;
        var applicable_flags = {
            wrong_tier_group: false,
        };
        var area_ranges = (_a = {},
            _a[Type.apprentice] = 72,
            _a[Type.journeyman] = 77,
            _a[Type.master] = 84,
            _a);
        applicable_flags.wrong_tier_group =
            node.props.world_area.area_level > area_ranges[this.type];
        return applicable_flags;
    };
    // I'm not sure how it determines the spawnweight
    // For example a map with no_monster_packs can still roll
    // a mod with spawnweight 0 for no_monster_packs if adjacent maps can roll it
    // I don't know which spawnweight it chooses though if
    // it can choose between multiple weights
    // for now we just search the adjacents if its 0 and pick first we find
    Sextant.prototype.findAdjacentWithSpawnweight = function (mod, node) {
        if (this.atlas == null) {
            throw new ContextUndefined('atlas');
        }
        return node.inSextantRange(this.atlas).find(function (other) {
            return mod.spawnweightFor(other) > 0;
        });
    };
    Sextant.prototype.isModSpawnableOn = function (mod, node) {
        var spawnable_flags = __assign({}, _super.prototype.isModSpawnableOn.call(this, mod, node), { no_adjacents_with_spawnweight: false });
        if (spawnable_flags.spawnweight_zero) {
            var adjacent_with_spawnweight = this.findAdjacentWithSpawnweight(mod, node);
            if (adjacent_with_spawnweight === undefined) {
                spawnable_flags.no_adjacents_with_spawnweight = true;
            }
            else {
                spawnable_flags.spawnweight_zero = false;
            }
        }
        return spawnable_flags;
    };
    Sextant.prototype.modsFor = function (node, whitelist) {
        var _this = this;
        if (whitelist === void 0) { whitelist = []; }
        if (this.atlas == null) {
            throw new ContextUndefined('atlas');
        }
        var blocked = Sextant.blockedMods(node, this.atlas);
        return _super.prototype.modsFor.call(this, node, whitelist)
            .filter(function (_a) {
            var mod = _a.mod;
            var is_blocked = blocked.includes(mod);
            return !is_blocked;
        })
            .map(function (_a) {
            var spawnable = _a.spawnable, spawnweight = _a.spawnweight, mod = _a.mod, details = __rest(_a, ["spawnable", "spawnweight", "mod"]);
            if (spawnable === undefined || spawnweight === undefined) {
                throw new Error('spawnable is undefined');
            }
            var spawnweight_with_adjacents = spawnweight;
            if (spawnweight === 0 && !spawnable.spawnweight_zero) {
                // see Sextant.isModSpawnableOn
                // we took the spawnweight from an adjacent node
                var adjacent_with_spawnweight = _this.findAdjacentWithSpawnweight(mod, node);
                // this shouldn't be happening spawnweight should be > 0
                // if the flag is not set. only exception is whitelist
                var corrupted_state = adjacent_with_spawnweight == null &&
                    !whitelist.includes('spawnweight_zero');
                if (corrupted_state) {
                    throw new CorruptedState('spawnweight should be > 0 if spawnweight_zero is false');
                }
                else if (adjacent_with_spawnweight != null) {
                    // we need to assure Flow that whitelist.includes didnt
                    // mutate adjacent_with_spawnweight although we wouldnt care about
                    // it because we dont care if it went from null > any
                    // we just need to be sure that it didnt go from any > null
                    // and this is given since it's the first check in the expression
                    spawnweight_with_adjacents = mod.spawnweightFor(adjacent_with_spawnweight);
                }
            }
            return __assign({ mod: mod,
                spawnable: spawnable, spawnweight: spawnweight_with_adjacents }, details);
        });
    };
    Sextant.type = Type;
    return Sextant;
}(Orb));
export default Sextant;
