var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import { anySet } from '../../util/Flags';
import ItemOrb from './ItemOrb';
import Mod from '../../mods/Mod';
import Alchemy from './Alchemy';
import Scouring from './Scouring';
/**
 * Essences guarantee at least exactly one mod depending on itemclass
 * or are not applicable. They should provide a mod for every equipment type.
 */
var Essence = /** @class */ (function (_super) {
    __extends(Essence, _super);
    /**
     *
     * @param props props of a specific essence (contains the guarenteed mod)
     * @param mods mods that are rolled onto the item after th guarenteed is applied
     */
    function Essence(props, mods) {
        var _this = _super.call(this, []) || this;
        _this.alchemy = Alchemy.build(mods);
        _this.props = props;
        return _this;
    }
    Essence.build = function (props, mods) {
        return new Essence(props, mods);
    };
    /**
     *  add the guarenteed mod and fill up the rest like alchemy
     */
    Essence.prototype.applyTo = function (item, options) {
        if (options === void 0) { options = {}; }
        var _a = options.force, force = _a === void 0 ? false : _a;
        var new_item = item;
        if (force || !anySet(this.applicableTo(item))) {
            // scour first if this reforges
            if (this.reforges()) {
                // essences ignore meta mods so dont use a scour implementation
                // and just blindly remove mods
                new_item = Essence.reforger.applyTo(new_item, {
                    ignore_meta_mods: true,
                    force: true,
                });
            }
            // 1. add guarenteed
            var guarenteed = this.chooseMod(new_item);
            if (guarenteed === undefined) {
                throw new Error("'" + new_item.baseitem.item_class + "' has no guarentedd mod");
            }
            new_item = new_item.rarity.set('rare').addMod(guarenteed);
            // 2. fill up like alch
            new_item = this.alchemy.applyTo(new_item, { force: true });
            return new_item;
        }
        return new_item;
    };
    /**
     * only one mod per itemclass
     */
    Essence.prototype.modsFor = function (item, whitelist) {
        if (whitelist === void 0) { whitelist = []; }
        var mod = this.chooseMod(item);
        if (mod === undefined) {
            return [];
        }
        return [{ mod: mod, spawnweight: Number.POSITIVE_INFINITY }];
    };
    /**
     * @returns Mod if the itemclass of the Item is eligible
     */
    Essence.prototype.chooseMod = function (item) {
        return this.modForItemclass(item.baseitem.item_class);
    };
    /**
     *
     * @param itemclass
     * @returns the guaranteed mod for the itemclass
     */
    Essence.prototype.modForItemclass = function (itemclass) {
        var mod_props = this.modPropsFor(itemclass);
        if (mod_props === undefined) {
            return undefined;
        }
        else {
            return new Mod(mod_props);
        }
    };
    /**
     * applicable if the essence guarantees a mod for the itemclass
     * and the rarity is either white or rare (only if essence can reforge)
     * @param item
     */
    Essence.prototype.applicableTo = function (item) {
        var applicable_flags = __assign({}, _super.prototype.applicableTo.call(this, item), { wrong_rarity: !item.rarity.isNormal(), wrong_itemclass: this.chooseMod(item) === undefined });
        if (this.reforges()) {
            applicable_flags.wrong_rarity =
                applicable_flags.wrong_rarity && !item.rarity.isRare();
        }
        return applicable_flags;
    };
    /**
     * @returns true if the essence can reforge (i.e. reroll) the item
     */
    Essence.prototype.reforges = function () {
        return this.props.tier > 5;
    };
    /**
     * @returns true if the essence is the best of its type
     */
    Essence.prototype.isTopTier = function () {
        return this.props.tier === 7;
    };
    /**
     * mapping from itemclass to mod prop in essence props
     * @param item_class
     */
    Essence.prototype.modPropsFor = function (item_class) {
        switch (item_class) {
            case 'Amulet':
                return this.props.amulet_mod;
            case 'Belt':
                return this.props.belt_mod;
            case 'Body Armour':
                return this.props.body_armour_mod;
            case 'Boots':
                return this.props.boots_mod;
            case 'Bow':
                return this.props.bow_mod;
            case 'Claw':
                return this.props.claw_mod;
            case 'Dagger':
                return this.props.dagger_mod;
            case 'Gloves':
                return this.props.gloves_mod;
            case 'Helmet':
                return this.props.helmet_mod;
            case 'One Hand Axe':
                return this.props.one_hand_axe_mod;
            case 'One Hand Mace':
                return this.props.one_hand_mace_mod;
            case 'One Hand Sword':
                return this.props.one_hand_sword_mod;
            case 'Thrusting One Hand Sword':
                return this.props.one_hand_thrusting_sword_mod;
            case 'Quiver':
                return this.props.quiver_mod;
            case 'Ring':
                return this.props.ring_mod;
            case 'Sceptre':
                return this.props.sceptre_mod;
            case 'Shield':
                return this.props.shield_mod;
            case 'Staff':
                return this.props.staff_mod;
            case 'Two Hand Axe':
                return this.props.two_hand_axe_mod;
            case 'Two Hand Mace':
                return this.props.two_hand_mace_mod;
            case 'Two Hand Sword':
                return this.props.two_hand_sword_mod;
            case 'Wand':
                return this.props.wand_mod;
            default:
                return undefined;
        }
    };
    Essence.reforger = new Scouring();
    return Essence;
}(ItemOrb));
export default Essence;
