var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import ViewOnlyOrb from '../ViewOnlyOrb';
import { Mod } from '../../mods';
import { anySet } from '../../util';
/**
 * Generator for all mods that can appear on items sold by Master vendors
 *
 * Remember that items sold from the vendor have a fixed item level and might
 * not be able to hold mods for higher item levels
 */
var MasterSignatureMods = /** @class */ (function (_super) {
    __extends(MasterSignatureMods, _super);
    function MasterSignatureMods() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MasterSignatureMods.build = function (mods) {
        return new MasterSignatureMods(mods
            .filter(MasterSignatureMods.isMasterSignature)
            .map(function (props) { return new Mod(props); }));
    };
    /**
     *
     * @param mod
     * @returns true if the mod can only be found on items bought from masters
     */
    MasterSignatureMods.isMasterSignature = function (mod) {
        return MasterSignatureMods.MOD_MASTER[mod.id] !== undefined;
    };
    /**
     * @param item
     * @param whitelist
     */
    MasterSignatureMods.prototype.modsFor = function (item, whitelist) {
        if (whitelist === void 0) { whitelist = []; }
        // basically ignore spawnweight
        var whitelist_spawnable = whitelist.concat('no_matching_tags', 'spawnweight_zero');
        return _super.prototype.modsFor.call(this, item, whitelist_spawnable)
            .map(function (details) {
            var mod = details.mod;
            var master = MasterSignatureMods.MOD_MASTER[mod.props.id];
            if (master === undefined) {
                // should not happen unless MOD_MASTER was changed
                throw new Error("no master found for '" + mod.props.id + "'");
            }
            var sellsItem = MasterSignatureMods.MASTER_ITEMCLASSES[master];
            if (sellsItem === undefined) {
                // should not happen unless MASTER_ITEMCLASSES was changed
                throw new Error("no itemclasses given for '" + master + "'");
            }
            return __assign({}, details, { applicable: __assign({}, details.applicable, { not_buyable: !sellsItem(item) }) });
        })
            .filter(function (_a) {
            var applicable = _a.applicable;
            return !anySet(applicable, whitelist_spawnable);
        });
    };
    /**
     * mapping from mod id to master id
     *
     * we need to know what master vendor can spawn the mods because the vendor
     * tells us what itemclasses can appear
     */
    MasterSignatureMods.MOD_MASTER = {
        // Leo
        AllDamageMasterVendorItem: 'pvp',
        // Zana
        MapQuantityAddedAsRarityVendorItem: 'str_dex_int',
        // Catarina
        LocalIncreaseSocketedSupportGemLevelIntMasterVendorItem: 'int',
        // Vorici
        LifeLeechSpeedDexIntMasterVendorItem: 'dex_int',
        // Haku
        SocketedGemQualityStrMasterVendorItem: 'str',
        // Tora
        BleedOnHitGainedDexMasterVendorItemUpdated_: 'dex',
        // Vagan
        AlwaysHitsStrDexMasterVendorItem: 'str_dex',
        // Elreon
        ReduceGlobalFlatManaCostStrIntMasterVendor: 'str_int',
    };
    /**
     * mapping from master id to a filter function that returns true if it could've
     * sold the given item
     *
     * TODO required level? Do master sell the best bases e.g. Hubris Circlets
     */
    MasterSignatureMods.MASTER_ITEMCLASSES = {
        // Leo
        pvp: function (item) { return ['Amulet', 'Ring', 'Belt'].includes(item.baseitem.item_class); },
        // Zana
        str_dex_int: function (item) { return ['Map'].includes(item.baseitem.item_class); },
        // Catarina
        int: function (item) {
            return ['Dagger', 'Wand', 'Sceptre'].includes(item.baseitem.item_class) ||
                isSpiritShield(item) ||
                isIntHelmet(item);
        },
        // Vorici
        dex_int: function (item) { return ['Gloves', 'Amulet'].includes(item.baseitem.item_class); },
        // Haku
        str: function (item) {
            return item.meta_data.isA('AbstractOneHandWeapon') ||
                item.meta_data.isA('AbstractTwoHandWeapon');
        },
        // Tora
        dex: function (item) { return ['Bow'].includes(item.baseitem.item_class); },
        // Vagan: 1H/2H melee weapon
        str_dex: function (item) {
            return [
                'One Hand Axe',
                'One Hand Mace',
                'One Hand Sword',
                'Thrusting One Hand Sword',
                'Sceptre',
                'Dagger',
                'Claw',
                'Two Hand Axe',
                'Two Hand Mace',
                'Two Hand Sword',
                'Staff',
            ].includes(item.baseitem.item_class);
        },
        // Elreon
        str_int: function (item) { return ['Ring', 'Amulet'].includes(item.baseitem.item_class); },
    };
    return MasterSignatureMods;
}(ViewOnlyOrb));
export default MasterSignatureMods;
function isSpiritShield(item) {
    return (item.baseitem.item_class === 'Shield' &&
        item.baseitem.tags.includes('focus'));
}
function isIntHelmet(item) {
    return (item.baseitem.item_class === 'Helmet' &&
        item.baseitem.tags.includes('int_armour'));
}
