import AtlasNode from '../containers/AtlasNode';
import Sextant from '../generators/Sextant';
import Mod from '../mods/Mod';
import { AtlasNodeProps } from '../schema';
import { GeneratorDetails } from '../generators/Generator';
export declare type HumanId = string;
export declare type AtlasNodes = Map<HumanId, AtlasNode>;
export interface Builder {
    nodes: AtlasNodes;
}
/**
 * immutable data structure for the atlas in Path of Exile
 *
 * main purpose is for reducer like usage in redux
 */
export default class Atlas {
    static buildLookupTable(atlas: AtlasNodeProps[]): AtlasNodes;
    static build(atlas: AtlasNodeProps[]): Atlas;
    static withBuilder(builder: Builder): Atlas;
    readonly nodes: AtlasNodes;
    constructor(nodes: AtlasNodes);
    asArray(): AtlasNode[];
    /**
     * wrapper for map get that ensures a node or throws
     */
    get(world_area_id: string): AtlasNode;
    builder(): Builder;
    /**
     * batch mutations
     *
     * if the returned object is strict equal to the prev one
     * it doesn't return a new copy
     */
    withMutations(mutate: (builder: Builder) => Builder): this;
    /**
     * removes mods on all maps
     *
     * always returns a new copy
     */
    reset(): this;
    addMod(mod: Mod, world_area_id: string): this;
    removeMod(mod: Mod, world_area_id: HumanId): this;
    mutateNode(world_area_id: string, mutate: (node: AtlasNode) => AtlasNode): this;
    applySextant(sextant: Sextant, world_area_id: string): this;
    modsFor(sextant: Sextant, world_area_id: string): Array<GeneratorDetails<Mod>>;
    blockedMods(world_area_id: string): Mod[];
    private prepareSextant;
}
